@@ -33,7 +33,7 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
 
-  if (!gst_element_register (plugin, "jpegenc", GST_RANK_NONE,
+  if (!gst_element_register (plugin, "jpegenc", GST_RANK_PRIMARY,
           GST_TYPE_JPEGENC))
     return FALSE;
 
