@@ -89,6 +89,12 @@
 GST_DEBUG_CATEGORY_STATIC (souphttpsrc_debug);
 #define GST_CAT_DEFAULT souphttpsrc_debug
 
+static const GstElementDetails gst_soup_http_src_details =
+GST_ELEMENT_DETAILS ("HTTP client source",
+    "Source/Network",
+    "Receive data as a client over the network via HTTP using SOUP",
+    "Wouter Cloetens <wouter@mind.be>");
+
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -120,7 +126,9 @@ enum
 
 static void gst_soup_http_src_uri_handler_init (gpointer g_iface,
     gpointer iface_data);
-static void gst_soup_http_src_finalize (GObject * gobject);
+static void gst_soup_http_src_init (GstSoupHTTPSrc * src,
+    GstSoupHTTPSrcClass * g_class);
+static void gst_soup_http_src_dispose (GObject * gobject);
 
 static void gst_soup_http_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -130,30 +138,44 @@ static void gst_soup_http_src_get_property (GObject * object, guint prop_id,
 static GstFlowReturn gst_soup_http_src_create (GstPushSrc * psrc,
     GstBuffer ** outbuf);
 static gboolean gst_soup_http_src_start (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_stop (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_get_size (GstBaseSrc * bsrc, guint64 * size);
+
 static gboolean gst_soup_http_src_is_seekable (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_do_seek (GstBaseSrc * bsrc,
     GstSegment * segment);
-static gboolean gst_soup_http_src_query (GstBaseSrc * bsrc, GstQuery * query);
 static gboolean gst_soup_http_src_unlock (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_unlock_stop (GstBaseSrc * bsrc);
+
 static gboolean gst_soup_http_src_set_location (GstSoupHTTPSrc * src,
     const gchar * uri);
 static gboolean gst_soup_http_src_set_proxy (GstSoupHTTPSrc * src,
     const gchar * uri);
+
 static char *gst_soup_http_src_unicodify (const char *str);
+
 static gboolean gst_soup_http_src_build_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_cancel_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_queue_message (GstSoupHTTPSrc * src);
+
 static gboolean gst_soup_http_src_add_range_header (GstSoupHTTPSrc * src,
     guint64 offset);
 static void gst_soup_http_src_session_unpause_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_session_pause_message (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_session_close (GstSoupHTTPSrc * src);
+
 static void gst_soup_http_src_parse_status (SoupMessage * msg,
     GstSoupHTTPSrc * src);
 static void gst_soup_http_src_chunk_free (gpointer gstbuf);
+
 static SoupBuffer *gst_soup_http_src_chunk_allocator (SoupMessage * msg,
     gsize max_len, gpointer user_data);
 static void gst_soup_http_src_got_chunk_cb (SoupMessage * msg,
@@ -193,19 +215,19 @@ gst_soup_http_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &srctemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&srctemplate));
 
-  gst_element_class_set_details_simple (element_class, "HTTP client source",
-      "Source/Network",
-      "Receive data as a client over the network via HTTP using SOUP",
-      "Wouter Cloetens <wouter@mind.be>");
+  gst_element_class_set_details (element_class, &gst_soup_http_src_details);
 }
 
 static void
 gst_soup_http_src_class_init (GstSoupHTTPSrcClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstBaseSrcClass *gstbasesrc_class;
+
   GstPushSrcClass *gstpushsrc_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -214,59 +236,56 @@ gst_soup_http_src_class_init (GstSoupHTTPSrcClass * klass)
 
   gobject_class->set_property = gst_soup_http_src_set_property;
   gobject_class->get_property = gst_soup_http_src_get_property;
-  gobject_class->finalize = gst_soup_http_src_finalize;
+  gobject_class->dispose = gst_soup_http_src_dispose;
 
   g_object_class_install_property (gobject_class,
       PROP_LOCATION,
       g_param_spec_string ("location", "Location",
-          "Location to read from", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Location to read from", "", G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       PROP_USER_AGENT,
       g_param_spec_string ("user-agent", "User-Agent",
           "Value of the User-Agent HTTP request header field",
-          DEFAULT_USER_AGENT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_USER_AGENT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       PROP_AUTOMATIC_REDIRECT,
       g_param_spec_boolean ("automatic-redirect", "automatic-redirect",
           "Automatically follow HTTP redirects (HTTP Status Code 3xx)",
-          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       PROP_PROXY,
       g_param_spec_string ("proxy", "Proxy",
-          "HTTP proxy server URI", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "HTTP proxy server URI", "", G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       PROP_USER_ID,
       g_param_spec_string ("user-id", "user-id",
           "HTTP location URI user id for authentication", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_USER_PW,
       g_param_spec_string ("user-pw", "user-pw",
           "HTTP location URI user password for authentication", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_PROXY_ID,
       g_param_spec_string ("proxy-id", "proxy-id",
-          "HTTP proxy URI user id for authentication", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "HTTP proxy URI user id for authentication", "", G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_PROXY_PW,
       g_param_spec_string ("proxy-pw", "proxy-pw",
           "HTTP proxy URI user password for authentication", "",
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_COOKIES,
       g_param_spec_boxed ("cookies", "Cookies", "HTTP request cookies",
-          G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_TYPE_STRV, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_IS_LIVE,
       g_param_spec_boolean ("is-live", "is-live", "Act like a live source",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_TIMEOUT,
       g_param_spec_uint ("timeout", "timeout",
           "Value in seconds to timeout a blocking I/O (0 = No timeout).", 0,
-          3600, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          3600, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_EXTRA_HEADERS,
       g_param_spec_boxed ("extra-headers", "Extra Headers",
           "Extra headers to append to the HTTP request",
-          GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_STRUCTURE, G_PARAM_READWRITE));
 
   /* icecast stuff */
   g_object_class_install_property (gobject_class,
@@ -274,29 +293,25 @@ gst_soup_http_src_class_init (GstSoupHTTPSrcClass * klass)
       g_param_spec_boolean ("iradio-mode",
           "iradio-mode",
           "Enable internet radio mode (extraction of shoutcast/icecast metadata)",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
       PROP_IRADIO_NAME,
       g_param_spec_string ("iradio-name",
-          "iradio-name", "Name of the stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "iradio-name", "Name of the stream", NULL, G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
       PROP_IRADIO_GENRE,
       g_param_spec_string ("iradio-genre",
-          "iradio-genre", "Genre of the stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "iradio-genre", "Genre of the stream", NULL, G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
       PROP_IRADIO_URL,
       g_param_spec_string ("iradio-url",
           "iradio-url",
-          "Homepage URL for radio stream", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "Homepage URL for radio stream", NULL, G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
       PROP_IRADIO_TITLE,
       g_param_spec_string ("iradio-title",
           "iradio-title",
-          "Name of currently playing song", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "Name of currently playing song", NULL, G_PARAM_READABLE));
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_soup_http_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_soup_http_src_stop);
@@ -307,31 +322,11 @@ gst_soup_http_src_class_init (GstSoupHTTPSrcClass * klass)
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_soup_http_src_is_seekable);
   gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR (gst_soup_http_src_do_seek);
-  gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_soup_http_src_query);
 
   gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_soup_http_src_create);
-}
 
-static void
-gst_soup_http_src_reset (GstSoupHTTPSrc * src)
-{
-  src->interrupted = FALSE;
-  src->retry = FALSE;
-  src->have_size = FALSE;
-  src->seekable = FALSE;
-  src->read_position = 0;
-  src->request_position = 0;
-  src->content_size = 0;
-
-  gst_caps_replace (&src->src_caps, NULL);
-  g_free (src->iradio_name);
-  src->iradio_name = NULL;
-  g_free (src->iradio_genre);
-  src->iradio_genre = NULL;
-  g_free (src->iradio_url);
-  src->iradio_url = NULL;
-  g_free (src->iradio_title);
-  src->iradio_title = NULL;
+  GST_DEBUG_CATEGORY_INIT (souphttpsrc_debug, "souphttpsrc", 0,
+      "SOUP HTTP Client Source");
 }
 
 static void
@@ -347,40 +342,67 @@ gst_soup_http_src_init (GstSoupHTTPSrc * src, GstSoupHTTPSrcClass * g_class)
   src->proxy_id = NULL;
   src->proxy_pw = NULL;
   src->cookies = NULL;
+  src->icy_caps = NULL;
   src->iradio_mode = FALSE;
+  src->iradio_name = NULL;
+  src->iradio_genre = NULL;
+  src->iradio_url = NULL;
+  src->iradio_title = NULL;
   src->loop = NULL;
   src->context = NULL;
   src->session = NULL;
   src->msg = NULL;
+  src->interrupted = FALSE;
+  src->retry = FALSE;
+  src->have_size = FALSE;
+  src->seekable = TRUE;
+  src->read_position = 0;
+  src->request_position = 0;
   proxy = g_getenv ("http_proxy");
   if (proxy && !gst_soup_http_src_set_proxy (src, proxy)) {
     GST_WARNING_OBJECT (src,
         "The proxy in the http_proxy env var (\"%s\") cannot be parsed.",
         proxy);
   }
-
-  gst_soup_http_src_reset (src);
 }
 
 static void
-gst_soup_http_src_finalize (GObject * gobject)
+gst_soup_http_src_dispose (GObject * gobject)
 {
   GstSoupHTTPSrc *src = GST_SOUP_HTTP_SRC (gobject);
 
-  GST_DEBUG_OBJECT (src, "finalize");
-
+  GST_DEBUG_OBJECT (src, "dispose");
   g_free (src->location);
+  src->location = NULL;
   g_free (src->user_agent);
+  src->user_agent = NULL;
   if (src->proxy != NULL) {
     soup_uri_free (src->proxy);
+    src->proxy = NULL;
   }
   g_free (src->user_id);
+  src->user_id = NULL;
   g_free (src->user_pw);
+  src->user_pw = NULL;
   g_free (src->proxy_id);
+  src->proxy_id = NULL;
   g_free (src->proxy_pw);
+  src->proxy_pw = NULL;
   g_strfreev (src->cookies);
+  g_free (src->iradio_name);
+  src->iradio_name = NULL;
+  g_free (src->iradio_genre);
+  src->iradio_genre = NULL;
+  g_free (src->iradio_url);
+  src->iradio_url = NULL;
+  g_free (src->iradio_title);
+  src->iradio_title = NULL;
+  if (src->icy_caps) {
+    gst_caps_unref (src->icy_caps);
+    src->icy_caps = NULL;
+  }
 
-  G_OBJECT_CLASS (parent_class)->finalize (gobject);
+  G_OBJECT_CLASS (parent_class)->dispose (gobject);
 }
 
 static void
@@ -498,7 +520,7 @@ gst_soup_http_src_get_property (GObject * object, guint prop_id,
       break;
     case PROP_PROXY:
       if (src->proxy == NULL)
-        g_value_set_static_string (value, "");
+        g_value_set_string (value, "");
       else {
         char *proxy = soup_uri_to_string (src->proxy, FALSE);
 
@@ -704,37 +726,25 @@ gst_soup_http_src_authenticate_cb (SoupSession * session, SoupMessage * msg,
   if (!retrying) {
     /* First time authentication only, if we fail and are called again with retry true fall through */
     if (msg->status_code == SOUP_STATUS_UNAUTHORIZED) {
-      if (src->user_id && src->user_pw)
-        soup_auth_authenticate (auth, src->user_id, src->user_pw);
+      soup_auth_authenticate (auth, src->user_id, src->user_pw);
     } else if (msg->status_code == SOUP_STATUS_PROXY_AUTHENTICATION_REQUIRED) {
-      if (src->proxy_id && src->proxy_pw)
-        soup_auth_authenticate (auth, src->proxy_id, src->proxy_pw);
+      soup_auth_authenticate (auth, src->proxy_id, src->proxy_pw);
     }
   }
 }
 
-static void
-gst_soup_http_src_headers_foreach (const gchar * name, const gchar * val,
-    gpointer src)
-{
-  GST_DEBUG_OBJECT (src, " %s: %s", name, val);
-}
-
 static void
 gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
 {
   const char *value;
+
   GstTagList *tag_list;
+
   GstBaseSrc *basesrc;
+
   guint64 newsize;
-  GHashTable *params = NULL;
 
-  GST_DEBUG_OBJECT (src, "got headers:");
-  soup_message_headers_foreach (msg->response_headers,
-      gst_soup_http_src_headers_foreach, src);
-
-  if (msg->status_code == 407 && src->proxy_id && src->proxy_pw)
-    return;
+  GST_DEBUG_OBJECT (src, "got headers");
 
   if (src->automatic_redirect && SOUP_STATUS_IS_REDIRECTION (msg->status_code)) {
     GST_DEBUG_OBJECT (src, "%u redirect to \"%s\"", msg->status_code,
@@ -755,7 +765,6 @@ gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
     if (!src->have_size || (src->content_size != newsize)) {
       src->content_size = newsize;
       src->have_size = TRUE;
-      src->seekable = TRUE;
       GST_DEBUG_OBJECT (src, "size = %" G_GUINT64_FORMAT, src->content_size);
 
       basesrc = GST_BASE_SRC_CAST (src);
@@ -777,50 +786,13 @@ gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
 
     GST_DEBUG_OBJECT (src, "icy-metaint: %s (parsed: %d)", value, icy_metaint);
     if (icy_metaint > 0) {
-      if (src->src_caps)
-        gst_caps_unref (src->src_caps);
+      if (src->icy_caps)
+        gst_caps_unref (src->icy_caps);
 
-      src->src_caps = gst_caps_new_simple ("application/x-icy",
+      src->icy_caps = gst_caps_new_simple ("application/x-icy",
           "metadata-interval", G_TYPE_INT, icy_metaint, NULL);
     }
   }
-  if ((value =
-          soup_message_headers_get_content_type (msg->response_headers,
-              &params)) != NULL) {
-    GST_DEBUG_OBJECT (src, "Content-Type: %s", value);
-    if (g_ascii_strcasecmp (value, "audio/L16") == 0) {
-      gint channels = 2;
-      gint rate = 44100;
-      char *param;
-
-      if (src->src_caps)
-        gst_caps_unref (src->src_caps);
-
-      param = g_hash_table_lookup (params, "channels");
-      if (param != NULL)
-        channels = atol (param);
-
-      param = g_hash_table_lookup (params, "rate");
-      if (param != NULL)
-        rate = atol (param);
-
-      src->src_caps = gst_caps_new_simple ("audio/x-raw-int",
-          "channels", G_TYPE_INT, channels,
-          "rate", G_TYPE_INT, rate,
-          "width", G_TYPE_INT, 16,
-          "depth", G_TYPE_INT, 16,
-          "signed", G_TYPE_BOOLEAN, TRUE,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN, NULL);
-    } else {
-      /* Set the Content-Type field on the caps */
-      if (src->src_caps)
-        gst_caps_set_simple (src->src_caps, "content-type", G_TYPE_STRING,
-            value, NULL);
-    }
-  }
-
-  if (params != NULL)
-    g_hash_table_destroy (params);
 
   if ((value =
           soup_message_headers_get (msg->response_headers,
@@ -869,9 +841,9 @@ gst_soup_http_src_got_headers_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
   if (src->ret == GST_FLOW_CUSTOM_ERROR &&
       src->read_position && msg->status_code != SOUP_STATUS_PARTIAL_CONTENT) {
     src->seekable = FALSE;
-    GST_ELEMENT_ERROR (src, RESOURCE, SEEK,
-        (_("Server does not support seeking.")),
-        ("Server does not accept Range HTTP header, URL: %s", src->location));
+    GST_ELEMENT_ERROR (src, RESOURCE, READ,
+        ("\"%s\": failed to seek; server does not accept Range HTTP header",
+            src->location), (NULL));
     src->ret = GST_FLOW_ERROR;
   }
 }
@@ -918,7 +890,6 @@ gst_soup_http_src_finished_cb (SoupMessage * msg, GstSoupHTTPSrc * src)
     src->ret = GST_FLOW_CUSTOM_ERROR;
   } else if (G_UNLIKELY (src->session_io_status !=
           GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_RUNNING)) {
-    /* FIXME: reason_phrase is not translated, add proper error message */
     GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
         ("%s", msg->reason_phrase),
         ("libsoup status code %d", msg->status_code));
@@ -961,10 +932,15 @@ gst_soup_http_src_chunk_allocator (SoupMessage * msg, gsize max_len,
     gpointer user_data)
 {
   GstSoupHTTPSrc *src = (GstSoupHTTPSrc *) user_data;
+
   GstBaseSrc *basesrc = GST_BASE_SRC_CAST (src);
+
   GstBuffer *gstbuf;
+
   SoupBuffer *soupbuf;
+
   gsize length;
+
   GstFlowReturn rc;
 
   if (max_len)
@@ -977,7 +953,7 @@ gst_soup_http_src_chunk_allocator (SoupMessage * msg, gsize max_len,
 
   rc = gst_pad_alloc_buffer (GST_BASE_SRC_PAD (basesrc),
       GST_BUFFER_OFFSET_NONE, length,
-      src->src_caps ? src->src_caps :
+      src->icy_caps ? src->icy_caps :
       GST_PAD_CAPS (GST_BASE_SRC_PAD (basesrc)), &gstbuf);
   if (G_UNLIKELY (rc != GST_FLOW_OK)) {
     /* Failed to allocate buffer. Stall SoupSession and return error code
@@ -998,6 +974,7 @@ gst_soup_http_src_got_chunk_cb (SoupMessage * msg, SoupBuffer * chunk,
     GstSoupHTTPSrc * src)
 {
   GstBaseSrc *basesrc;
+
   guint64 new_position;
 
   if (G_UNLIKELY (msg != src->msg)) {
@@ -1015,16 +992,14 @@ gst_soup_http_src_got_chunk_cb (SoupMessage * msg, SoupBuffer * chunk,
 
   /* Extract the GstBuffer from the SoupBuffer and set its fields. */
   *src->outbuf = GST_BUFFER_CAST (soup_buffer_get_owner (chunk));
-
+  gst_buffer_ref (*src->outbuf);
   GST_BUFFER_SIZE (*src->outbuf) = chunk->length;
   GST_BUFFER_OFFSET (*src->outbuf) = basesrc->segment.last_stop;
 
   gst_buffer_set_caps (*src->outbuf,
-      (src->src_caps) ? src->src_caps :
+      (src->icy_caps) ? src->icy_caps :
       GST_PAD_CAPS (GST_BASE_SRC_PAD (basesrc)));
 
-  gst_buffer_ref (*src->outbuf);
-
   new_position = src->read_position + chunk->length;
   if (G_LIKELY (src->request_position == src->read_position))
     src->request_position = new_position;
@@ -1064,42 +1039,36 @@ gst_soup_http_src_response_cb (SoupSession * session, SoupMessage * msg,
   g_main_loop_quit (src->loop);
 }
 
-#define SOUP_HTTP_SRC_ERROR(src,soup_msg,cat,code,error_message)     \
-  GST_ELEMENT_ERROR ((src), cat, code, ("%s", error_message),        \
-      ("%s (%d), URL: %s", (soup_msg)->reason_phrase,                \
-          (soup_msg)->status_code, (src)->location));
-
 static void
 gst_soup_http_src_parse_status (SoupMessage * msg, GstSoupHTTPSrc * src)
 {
   if (SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
     switch (msg->status_code) {
       case SOUP_STATUS_CANT_RESOLVE:
+        GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+            ("\"%s\": %s", src->location, msg->reason_phrase),
+            ("libsoup status code %d", msg->status_code));
+        src->ret = GST_FLOW_ERROR;
+        break;
       case SOUP_STATUS_CANT_RESOLVE_PROXY:
-        SOUP_HTTP_SRC_ERROR (src, msg, RESOURCE, NOT_FOUND,
-            _("Could not resolve server name."));
+        GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+            ("%s", msg->reason_phrase),
+            ("libsoup status code %d", msg->status_code));
         src->ret = GST_FLOW_ERROR;
         break;
       case SOUP_STATUS_CANT_CONNECT:
       case SOUP_STATUS_CANT_CONNECT_PROXY:
-        SOUP_HTTP_SRC_ERROR (src, msg, RESOURCE, OPEN_READ,
-            _("Could not establish connection to server."));
-        src->ret = GST_FLOW_ERROR;
-        break;
       case SOUP_STATUS_SSL_FAILED:
-        SOUP_HTTP_SRC_ERROR (src, msg, RESOURCE, OPEN_READ,
-            _("Secure connection setup failed."));
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+            ("\"%s\": %s", src->location, msg->reason_phrase),
+            ("libsoup status code %d", msg->status_code));
         src->ret = GST_FLOW_ERROR;
         break;
       case SOUP_STATUS_IO_ERROR:
-        SOUP_HTTP_SRC_ERROR (src, msg, RESOURCE, READ,
-            _("A network error occured, or the server closed the connection "
-                "unexpectedly."));
-        src->ret = GST_FLOW_ERROR;
-        break;
       case SOUP_STATUS_MALFORMED:
-        SOUP_HTTP_SRC_ERROR (src, msg, RESOURCE, READ,
-            _("Server sent bad data."));
+        GST_ELEMENT_ERROR (src, RESOURCE, READ,
+            ("\"%s\": %s", src->location, msg->reason_phrase),
+            ("libsoup status code %d", msg->status_code));
         src->ret = GST_FLOW_ERROR;
         break;
       case SOUP_STATUS_CANCELLED:
@@ -1110,13 +1079,9 @@ gst_soup_http_src_parse_status (SoupMessage * msg, GstSoupHTTPSrc * src)
       SOUP_STATUS_IS_REDIRECTION (msg->status_code) ||
       SOUP_STATUS_IS_SERVER_ERROR (msg->status_code)) {
     /* Report HTTP error. */
-    /* FIXME: reason_phrase is not translated and not suitable for user
-     * error dialog according to libsoup documentation.
-     * FIXME: error code (OPEN_READ vs. READ) should depend on http status? */
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        ("%s", msg->reason_phrase),
-        ("%s (%d), URL: %s", msg->reason_phrase, msg->status_code,
-            src->location));
+        ("\"%s\": %s", src->location, msg->reason_phrase),
+        ("%d %s", msg->status_code, msg->reason_phrase));
     src->ret = GST_FLOW_ERROR;
   }
 }
@@ -1127,7 +1092,7 @@ gst_soup_http_src_build_message (GstSoupHTTPSrc * src)
   src->msg = soup_message_new (SOUP_METHOD_GET, src->location);
   if (!src->msg) {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-        ("Error parsing URL."), ("URL: %s", src->location));
+        (NULL), ("Error parsing URL \"%s\"", src->location));
     return FALSE;
   }
   src->session_io_status = GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_IDLE;
@@ -1165,10 +1130,6 @@ gst_soup_http_src_build_message (GstSoupHTTPSrc * src)
 
   gst_soup_http_src_add_extra_headers (src);
 
-  GST_DEBUG_OBJECT (src, "request headers:");
-  soup_message_headers_foreach (src->msg->request_headers,
-      gst_soup_http_src_headers_foreach, src);
-
   return TRUE;
 }
 
@@ -1180,11 +1141,7 @@ gst_soup_http_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   src = GST_SOUP_HTTP_SRC (psrc);
 
   if (src->msg && (src->request_position != src->read_position)) {
-    if (src->content_size != 0 && src->request_position >= src->content_size) {
-      GST_WARNING_OBJECT (src, "Seeking behind the end of file -- EOS");
-      return GST_FLOW_UNEXPECTED;
-    } else if (src->session_io_status ==
-        GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_IDLE) {
+    if (src->session_io_status == GST_SOUP_HTTP_SRC_SESSION_IO_STATUS_IDLE) {
       gst_soup_http_src_add_range_header (src, src->request_position);
     } else {
       GST_DEBUG_OBJECT (src, "Seek from position %" G_GUINT64_FORMAT
@@ -1248,8 +1205,8 @@ gst_soup_http_src_start (GstBaseSrc * bsrc)
   GST_DEBUG_OBJECT (src, "start(\"%s\")", src->location);
 
   if (!src->location) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (_("No URL set.")),
-        ("Missing location property"));
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+        (NULL), ("Missing location property"));
     return FALSE;
   }
 
@@ -1310,7 +1267,6 @@ gst_soup_http_src_stop (GstBaseSrc * bsrc)
     src->extra_headers = NULL;
   }
 
-  gst_soup_http_src_reset (src);
   return TRUE;
 }
 
@@ -1374,52 +1330,17 @@ gst_soup_http_src_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
 
   GST_DEBUG_OBJECT (src, "do_seek(%" G_GUINT64_FORMAT ")", segment->start);
 
-  if (src->read_position == segment->start) {
-    GST_DEBUG_OBJECT (src, "Seeking to current read position");
+  if (src->read_position == segment->start)
     return TRUE;
-  }
 
-  if (!src->seekable) {
-    GST_WARNING_OBJECT (src, "Not seekable");
+  if (!src->seekable)
     return FALSE;
-  }
-
-  if (segment->rate < 0.0 || segment->format != GST_FORMAT_BYTES) {
-    GST_WARNING_OBJECT (src, "Invalid seek segment");
-    return FALSE;
-  }
-
-  if (src->content_size != 0 && segment->start >= src->content_size) {
-    GST_WARNING_OBJECT (src, "Seeking behind end of file, will go to EOS soon");
-  }
 
   /* Wait for create() to handle the jump in offset. */
   src->request_position = segment->start;
   return TRUE;
 }
 
-static gboolean
-gst_soup_http_src_query (GstBaseSrc * bsrc, GstQuery * query)
-{
-  GstSoupHTTPSrc *src = GST_SOUP_HTTP_SRC (bsrc);
-  gboolean ret;
-
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_URI:
-      gst_query_set_uri (query, src->location);
-      ret = TRUE;
-      break;
-    default:
-      ret = FALSE;
-      break;
-  }
-
-  if (!ret)
-    ret = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
-
-  return ret;
-}
-
 static gboolean
 gst_soup_http_src_set_location (GstSoupHTTPSrc * src, const gchar * uri)
 {
@@ -1460,8 +1381,8 @@ gst_soup_http_src_uri_get_type (void)
 static gchar **
 gst_soup_http_src_uri_get_protocols (void)
 {
-  static const gchar *protocols[] = { "http", "https", NULL };
-  return (gchar **) protocols;
+  static gchar *protocols[] = { "http", "https", NULL };
+  return protocols;
 }
 
 static const gchar *
@@ -1490,3 +1411,16 @@ gst_soup_http_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
   iface->get_uri = gst_soup_http_src_uri_get_uri;
   iface->set_uri = gst_soup_http_src_uri_set_uri;
 }
+
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "souphttpsrc", GST_RANK_PRIMARY,
+      GST_TYPE_SOUP_HTTP_SRC);
+}
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "soup",
+    "libsoup HTTP client src",
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
