@@ -22,30 +22,25 @@
  * SECTION:element-id3v2mux
  * @see_also: #GstID3Demux, #GstTagSetter
  *
- * <refsect2>
- * <para>
  * This element adds ID3v2 tags to the beginning of a stream using the taglib
  * library. More precisely, the tags written are ID3 version 2.4.0 tags (which
  * means in practice that some hardware players or outdated programs might not
  * be able to read them properly).
- * </para>
- * <para>
+ *
  * Applications can set the tags to write using the #GstTagSetter interface.
  * Tags sent by upstream elements will be picked up automatically (and merged
  * according to the merge mode set via the tag setter interface).
- * </para>
- * <para>
- * Here is a simple pipeline that transcodes a file from Ogg/Vorbis to mp3
- * format with an ID3v2 that contains the same as the the Ogg/Vorbis file:
- * <programlisting>
+ *
+ * <refsect2>
+ * <title>Example pipelines</title>
+ * |[
  * gst-launch -v filesrc location=foo.ogg ! decodebin ! audioconvert ! lame ! id3v2mux ! filesink location=foo.mp3
- * </programlisting>
- * Make sure the Ogg/Vorbis file actually has comments to preserve.
- * You can verify the tags were written using:
- * <programlisting>
+ * ]| A pipeline that transcodes a file from Ogg/Vorbis to mp3 format with an
+ * ID3v2 that contains the same as the the Ogg/Vorbis file. Make sure the
+ * Ogg/Vorbis file actually has comments to preserve.
+ * |[
  * gst-launch -m filesrc location=foo.mp3 ! id3demux ! fakesink silent=TRUE 2&gt; /dev/null | grep taglist
- * </programlisting>
- * </para>
+ * ]| Verify that tags have been written.
  * </refsect2>
  */
 
@@ -60,7 +55,10 @@
 #include <textidentificationframe.h>
 #include <uniquefileidentifierframe.h>
 #include <attachedpictureframe.h>
+#include <relativevolumeframe.h>
 #include <commentsframe.h>
+#include <unknownframe.h>
+#include <id3v2synchdata.h>
 #include <id3v2tag.h>
 #include <gst/tag/tag.h>
 
@@ -69,12 +67,6 @@ using namespace TagLib;
 GST_DEBUG_CATEGORY_STATIC (gst_id3v2_mux_debug);
 #define GST_CAT_DEFAULT gst_id3v2_mux_debug
 
-static const GstElementDetails gst_id3v2_mux_details =
-GST_ELEMENT_DETAILS ("TagLib-based ID3v2 Muxer",
-    "Formatter/Metadata",
-    "Adds an ID3v2 header to the beginning of MP3 files using taglib",
-    "Christophe Fergeau <teuf@gnome.org>");
-
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -95,7 +87,10 @@ gst_id3v2_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details (element_class, &gst_id3v2_mux_details);
+  gst_element_class_set_details_simple (element_class,
+      "TagLib-based ID3v2 Muxer", "Formatter/Metadata",
+      "Adds an ID3v2 header to the beginning of MP3 files using taglib",
+      "Christophe Fergeau <teuf@gnome.org>");
 
   GST_DEBUG_CATEGORY_INIT (gst_id3v2_mux_debug, "id3v2mux", 0,
       "taglib-based ID3v2 tag muxer");
@@ -322,7 +317,14 @@ add_musicbrainz_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
     GST_TAG_MUSICBRAINZ_ALBUMID, "MusicBrainz Album Id", "musicbrainz_albumid"}, {
     GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "MusicBrainz Album Artist Id",
           "musicbrainz_albumartistid"}, {
-    GST_TAG_MUSICBRAINZ_TRMID, "MusicBrainz TRM Id", "musicbrainz_trmid"}
+    GST_TAG_MUSICBRAINZ_TRMID, "MusicBrainz TRM Id", "musicbrainz_trmid"}, {
+    GST_TAG_CDDA_MUSICBRAINZ_DISCID, "MusicBrainz DiscID",
+          "musicbrainz_discid"}, {
+      /* the following one is more or less made up, there seems to be little
+       * evidence that any popular application is actually putting this info
+       * into TXXX frames; the first one comes from a musicbrainz wiki 'proposed
+       * tags' page, the second one is analogue to the vorbis/ape/flac tag. */
+    GST_TAG_CDDA_CDDB_DISCID, "CDDB DiscID", "discid"}
   };
   guint i, idx;
 
@@ -518,6 +520,112 @@ add_text_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
   }
 }
 
+static void
+add_uri_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
+    const gchar * tag, guint num_tags, const gchar * frame_id)
+{
+  gchar *url = NULL;
+
+  g_assert (frame_id != NULL);
+
+  /* URI tags are limited to one of each per taglist */
+  if (gst_tag_list_get_string_index (list, tag, 0, &url) && url != NULL) {
+    guint url_len;
+
+    url_len = strlen (url);
+    if (url_len > 0 && gst_uri_is_valid (url)) {
+      ID3v2::FrameFactory * factory = ID3v2::FrameFactory::instance ();
+      ID3v2::Frame * frame;
+      char *data;
+
+      data = g_new0 (char, 10 + url_len);
+
+      memcpy (data, frame_id, 4);
+      memcpy (data + 4, ID3v2::SynchData::fromUInt (url_len).data (), 4);
+      memcpy (data + 10, url, url_len);
+      ByteVector bytes (data, 10 + url_len);
+
+      g_free (data);
+
+      frame = factory->createFrame (bytes, (TagLib::uint) 4);
+      if (frame) {
+        id3v2tag->addFrame (frame);
+
+        GST_LOG ("%s: %s = '%s'", frame_id, tag, url);
+      }
+    } else {
+      GST_WARNING ("Tag %s does not contain a valid URI (%s)", tag, url);
+    }
+
+    g_free (url);
+  }
+}
+
+static void
+add_relative_volume_tag (ID3v2::Tag * id3v2tag, const GstTagList * list,
+    const gchar * tag, guint num_tags, const gchar * frame_id)
+{
+  const char *gain_tag_name;
+  const char *peak_tag_name;
+  gdouble peak_val;
+  gdouble gain_val;
+  ID3v2::RelativeVolumeFrame * frame;
+
+  frame = new ID3v2::RelativeVolumeFrame ();
+
+  /* figure out tag names and the identification string to use */
+  if (strcmp (tag, GST_TAG_TRACK_PEAK) == 0 ||
+      strcmp (tag, GST_TAG_TRACK_GAIN) == 0) {
+    gain_tag_name = GST_TAG_TRACK_GAIN;
+    peak_tag_name = GST_TAG_TRACK_PEAK;
+    frame->setIdentification ("track");
+    GST_DEBUG ("adding track relative-volume frame");
+  } else {
+    gain_tag_name = GST_TAG_ALBUM_GAIN;
+    peak_tag_name = GST_TAG_ALBUM_PEAK;
+    frame->setIdentification ("album");
+    GST_DEBUG ("adding album relative-volume frame");
+  }
+  
+  /* find the value for the paired tag (gain, if this is peak, and
+   * vice versa).  if both tags exist, only write the frame when
+   * we're processing the peak tag.
+   */
+  if (strcmp (tag, GST_TAG_TRACK_PEAK) == 0 ||
+      strcmp (tag, GST_TAG_ALBUM_PEAK) == 0) {
+    ID3v2::RelativeVolumeFrame::PeakVolume encoded_peak;
+    short peak_int;
+
+    gst_tag_list_get_double (list, tag, &peak_val);
+
+    if (gst_tag_list_get_tag_size (list, gain_tag_name) > 0) {
+      gst_tag_list_get_double (list, gain_tag_name, &gain_val);
+      GST_DEBUG ("setting volume adjustment %g", gain_val);
+      frame->setVolumeAdjustment (gain_val);
+    }
+
+    /* copying mutagen: always write as 16 bits for sanity. */
+    peak_int = (short)(peak_val * G_MAXSHORT);
+    encoded_peak.bitsRepresentingPeak = 16;
+    encoded_peak.peakVolume = ByteVector::fromShort(peak_int, true);
+    GST_DEBUG ("setting peak value %g", peak_val);
+    frame->setPeakVolume(encoded_peak);
+
+  } else {
+    gst_tag_list_get_double (list, tag, &gain_val);
+    GST_DEBUG ("setting volume adjustment %g", gain_val);
+    frame->setVolumeAdjustment (gain_val);
+
+    if (gst_tag_list_get_tag_size (list, peak_tag_name) != 0) {
+      GST_DEBUG ("both gain and peak tags exist, not adding frame this time around");
+      delete frame;
+      return;
+    }
+  }
+
+  id3v2tag->addFrame (frame);
+}
+
 /* id3demux produces these for frames it cannot parse */
 #define GST_ID3_DEMUX_TAG_ID3V2_FRAME "private-id3v2-frame"
 
@@ -532,6 +640,7 @@ static const struct
   GST_TAG_TITLE, add_text_tag, "TIT2"}, {
   GST_TAG_ALBUM, add_text_tag, "TALB"}, {
   GST_TAG_COPYRIGHT, add_text_tag, "TCOP"}, {
+  GST_TAG_COMPOSER, add_text_tag, "TCOM"}, {
   GST_TAG_GENRE, add_text_tag, "TCON"}, {
   GST_TAG_COMMENT, add_comment_tag, ""}, {
   GST_TAG_EXTENDED_COMMENT, add_comment_tag, ""}, {
@@ -543,14 +652,24 @@ static const struct
   GST_TAG_MUSICBRAINZ_ALBUMID, add_musicbrainz_tag, "\001"}, {
   GST_TAG_MUSICBRAINZ_ALBUMARTISTID, add_musicbrainz_tag, "\002"}, {
   GST_TAG_MUSICBRAINZ_TRMID, add_musicbrainz_tag, "\003"}, {
+  GST_TAG_CDDA_MUSICBRAINZ_DISCID, add_musicbrainz_tag, "\004"}, {
+  GST_TAG_CDDA_CDDB_DISCID, add_musicbrainz_tag, "\005"}, {
   GST_TAG_MUSICBRAINZ_TRACKID, add_unique_file_id_tag, ""}, {
-  GST_TAG_MUSICBRAINZ_SORTNAME, add_text_tag, "TSOP"}, {
+  GST_TAG_ARTIST_SORTNAME, add_text_tag, "TSOP"}, {
+  GST_TAG_ALBUM_SORTNAME, add_text_tag, "TSOA"}, {
+  GST_TAG_TITLE_SORTNAME, add_text_tag, "TSOT"}, {
   GST_TAG_TRACK_NUMBER, add_count_or_num_tag, "TRCK"}, {
   GST_TAG_TRACK_COUNT, add_count_or_num_tag, "TRCK"}, {
   GST_TAG_ALBUM_VOLUME_NUMBER, add_count_or_num_tag, "TPOS"}, {
   GST_TAG_ALBUM_VOLUME_COUNT, add_count_or_num_tag, "TPOS"}, {
   GST_TAG_ENCODER, add_encoder_tag, ""}, {
-  GST_TAG_ENCODER_VERSION, add_encoder_tag, ""}
+  GST_TAG_ENCODER_VERSION, add_encoder_tag, ""}, {
+  GST_TAG_COPYRIGHT_URI, add_uri_tag, "WCOP"}, {
+  GST_TAG_LICENSE_URI, add_uri_tag, "WCOP"}, {
+  GST_TAG_TRACK_PEAK, add_relative_volume_tag, ""}, {
+  GST_TAG_TRACK_GAIN, add_relative_volume_tag, ""}, {
+  GST_TAG_ALBUM_PEAK, add_relative_volume_tag, ""}, {
+  GST_TAG_ALBUM_GAIN, add_relative_volume_tag, ""}
 };
 
 
