@@ -39,7 +39,6 @@
 #include "gstflvmux.h"
 
 #include <string.h>
-#include <gst/base/gstbytereader.h>
 
 static GstStaticPadTemplate flv_sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -69,9 +68,6 @@ GST_BOILERPLATE (GstFLVDemux, gst_flv_demux, GstElement, GST_TYPE_ELEMENT);
 /* 1 byte of tag type + 3 bytes of tag data size */
 #define FLV_TAG_TYPE_SIZE 4
 
-static gboolean flv_demux_handle_seek_push (GstFLVDemux * demux,
-    GstEvent * event);
-
 static void
 gst_flv_demux_flush (GstFLVDemux * demux, gboolean discont)
 {
@@ -84,8 +80,8 @@ gst_flv_demux_flush (GstFLVDemux * demux, gboolean discont)
 
   demux->flushing = FALSE;
 
-  /* Only in push mode and if we're not during a seek */
-  if (!demux->random_access && demux->state != FLV_STATE_SEEK) {
+  /* Only in push mode */
+  if (!demux->random_access) {
     /* After a flush we expect a tag_type */
     demux->state = FLV_STATE_TAG_TYPE;
     /* We reset the offset and will get one from first push */
@@ -116,12 +112,6 @@ gst_flv_demux_cleanup (GstFLVDemux * demux)
   demux->push_tags = FALSE;
   demux->got_par = FALSE;
 
-  demux->indexed = FALSE;
-
-  demux->audio_start = demux->video_start = GST_CLOCK_TIME_NONE;
-
-  demux->no_more_pads = FALSE;
-
   gst_segment_init (&demux->segment, GST_FORMAT_TIME);
 
   demux->w = demux->h = 0;
@@ -177,29 +167,6 @@ gst_flv_demux_cleanup (GstFLVDemux * demux)
   }
 }
 
-/*
- * Create and push a flushing seek event upstream
- */
-static gboolean
-flv_demux_seek_to_offset (GstFLVDemux * demux, guint64 offset)
-{
-  GstEvent *event;
-  gboolean res = 0;
-
-  GST_DEBUG_OBJECT (demux, "Seeking to %" G_GUINT64_FORMAT, offset);
-
-  event =
-      gst_event_new_seek (1.0, GST_FORMAT_BYTES,
-      GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE, GST_SEEK_TYPE_SET, offset,
-      GST_SEEK_TYPE_NONE, -1);
-
-  res = gst_pad_push_event (demux->sinkpad, event);
-
-  if (res)
-    demux->offset = offset;
-  return res;
-}
-
 static GstFlowReturn
 gst_flv_demux_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -224,13 +191,6 @@ gst_flv_demux_chain (GstPad * pad, GstBuffer * buffer)
 
   gst_adapter_push (demux->adapter, buffer);
 
-  if (demux->seeking) {
-    demux->state = FLV_STATE_SEEK;
-    GST_OBJECT_LOCK (demux);
-    demux->seeking = FALSE;
-    GST_OBJECT_UNLOCK (demux);
-  }
-
 parse:
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
     if (ret == GST_FLOW_NOT_LINKED && (demux->audio_linked
@@ -282,11 +242,6 @@ parse:
         gst_buffer_unref (buffer);
         demux->offset += FLV_TAG_TYPE_SIZE;
 
-        /* last tag is not an index => no index/don't know where the index is
-         * seek back to the beginning */
-        if (demux->seek_event && demux->state != FLV_STATE_TAG_SCRIPT)
-          goto no_index;
-
         goto parse;
       } else {
         goto beach;
@@ -341,67 +296,11 @@ parse:
         demux->offset += demux->tag_size;
 
         demux->state = FLV_STATE_TAG_TYPE;
-
-        /* if there's a seek event we're here for the index so if we don't have it
-         * we seek back to the beginning */
-        if (demux->seek_event) {
-          if (demux->indexed)
-            demux->state = FLV_STATE_SEEK;
-          else
-            goto no_index;
-        }
-
         goto parse;
       } else {
         goto beach;
       }
     }
-    case FLV_STATE_SEEK:
-    {
-      GstEvent *event;
-
-      ret = GST_FLOW_OK;
-
-      if (!demux->indexed) {
-        if (demux->offset == demux->file_size - sizeof (guint32)) {
-          GstBuffer *buffer =
-              gst_adapter_take_buffer (demux->adapter, sizeof (guint32));
-          GstByteReader *reader = gst_byte_reader_new_from_buffer (buffer);
-          guint64 seek_offset;
-
-          if (!gst_adapter_available (demux->adapter) >= sizeof (guint32)) {
-            /* error */
-          }
-
-          seek_offset =
-              demux->file_size - sizeof (guint32) -
-              gst_byte_reader_peek_uint32_be_unchecked (reader);
-          gst_byte_reader_free (reader);
-          gst_buffer_unref (buffer);
-
-          GST_INFO_OBJECT (demux,
-              "Seeking to beginning of last tag at %" G_GUINT64_FORMAT,
-              seek_offset);
-          demux->state = FLV_STATE_TAG_TYPE;
-          flv_demux_seek_to_offset (demux, seek_offset);
-          goto beach;
-        } else
-          goto no_index;
-      }
-
-      GST_OBJECT_LOCK (demux);
-      event = demux->seek_event;
-      demux->seek_event = NULL;
-      GST_OBJECT_UNLOCK (demux);
-
-      /* calculate and perform seek */
-      if (!flv_demux_handle_seek_push (demux, event))
-        goto seek_failed;
-
-      gst_event_unref (event);
-      demux->state = FLV_STATE_TAG_TYPE;
-      goto beach;
-    }
     default:
       GST_DEBUG_OBJECT (demux, "unexpected demuxer state");
   }
@@ -417,26 +316,6 @@ beach:
   gst_object_unref (demux);
 
   return ret;
-
-/* ERRORS */
-no_index:
-  {
-    GST_OBJECT_LOCK (demux);
-    demux->seeking = FALSE;
-    gst_event_unref (demux->seek_event);
-    demux->seek_event = NULL;
-    GST_OBJECT_UNLOCK (demux);
-    GST_WARNING_OBJECT (demux,
-        "failed to find an index, seeking back to beginning");
-    flv_demux_seek_to_offset (demux, 0);
-    return GST_FLOW_OK;
-  }
-seek_failed:
-  {
-    GST_ELEMENT_ERROR (demux, STREAM, DEMUX, (NULL), ("seek failed"));
-    return GST_FLOW_ERROR;
-  }
-
 }
 
 static GstFlowReturn
@@ -589,17 +468,19 @@ gst_flv_demux_create_index (GstFLVDemux * demux)
   size_t tag_size;
   guint64 old_offset;
   GstBuffer *buffer;
+  GstFlowReturn ret;
 
-  if (G_UNLIKELY (!gst_pad_query_peer_duration (demux->sinkpad, &fmt, &size) ||
-          fmt != GST_FORMAT_BYTES))
+  if (!gst_pad_query_peer_duration (demux->sinkpad, &fmt, &size) ||
+      fmt != GST_FORMAT_BYTES)
     return;
 
   old_offset = demux->offset;
 
-  while (gst_flv_demux_pull_range (demux, demux->sinkpad, demux->offset, 12,
-          &buffer) == GST_FLOW_OK) {
-    if (G_UNLIKELY (gst_flv_parse_tag_timestamp (demux, buffer,
-                &tag_size) == GST_CLOCK_TIME_NONE)) {
+  while ((ret =
+          gst_flv_demux_pull_range (demux, demux->sinkpad, demux->offset, 12,
+              &buffer)) == GST_FLOW_OK) {
+    if (gst_flv_parse_tag_timestamp (demux, buffer,
+            &tag_size) == GST_CLOCK_TIME_NONE) {
       gst_buffer_unref (buffer);
       break;
     }
@@ -690,11 +571,7 @@ pause:
     if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
       if (ret == GST_FLOW_UNEXPECTED) {
         /* perform EOS logic */
-        if (!demux->no_more_pads) {
-          gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
-          demux->no_more_pads = TRUE;
-        }
-
+        gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
         if (demux->segment.flags & GST_SEEK_FLAG_SEGMENT) {
           gint64 stop;
 
@@ -717,11 +594,7 @@ pause:
           }
         } else {
           /* normal playback, send EOS to all linked pads */
-          if (!demux->no_more_pads) {
-            gst_element_no_more_pads (GST_ELEMENT (demux));
-            demux->no_more_pads = TRUE;
-          }
-
+          gst_element_no_more_pads (GST_ELEMENT (demux));
           GST_LOG_OBJECT (demux, "Sending EOS, at end of stream");
           if (!gst_flv_demux_push_src_event (demux, gst_event_new_eos ()))
             GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
@@ -781,7 +654,7 @@ gst_flv_demux_find_offset (GstFLVDemux * demux, GstSegment * segment)
 }
 
 static gboolean
-flv_demux_handle_seek_push (GstFLVDemux * demux, GstEvent * event)
+gst_flv_demux_handle_seek_push (GstFLVDemux * demux, GstEvent * event)
 {
   GstFormat format;
   GstSeekFlags flags;
@@ -798,7 +671,6 @@ flv_demux_handle_seek_push (GstFLVDemux * demux, GstEvent * event)
     goto wrong_format;
 
   flush = !!(flags & GST_SEEK_FLAG_FLUSH);
-  /* FIXME : the keyframe flag is never used ! */
   keyframe = !!(flags & GST_SEEK_FLAG_KEY_UNIT);
 
   /* Work on a copy until we are sure the seek succeeded. */
@@ -863,64 +735,6 @@ wrong_format:
   }
 }
 
-static gboolean
-gst_flv_demux_handle_seek_push (GstFLVDemux * demux, GstEvent * event)
-{
-  if (!demux->indexed) {
-    guint64 seek_offset;
-    gboolean building_index;
-    GstFormat fmt;
-
-    GST_OBJECT_LOCK (demux);
-    /* handle the seek in the chain function */
-    demux->seeking = TRUE;
-    demux->state = FLV_STATE_SEEK;
-
-    /* copy the event */
-    if (demux->seek_event)
-      gst_event_unref (demux->seek_event);
-    demux->seek_event = gst_event_ref (event);
-
-    /* set the building_index flag so that only one thread can setup the
-     * structures for index seeking. */
-    building_index = demux->building_index;
-    if (!building_index) {
-      demux->building_index = TRUE;
-      fmt = GST_FORMAT_BYTES;
-      if (!demux->file_size
-          && !gst_pad_query_peer_duration (demux->sinkpad, &fmt,
-              &demux->file_size)) {
-        GST_WARNING_OBJECT (demux,
-            "Cannot obtain file size - %" G_GINT64_FORMAT ", format %u",
-            demux->file_size, fmt);
-        GST_OBJECT_UNLOCK (demux);
-        return FALSE;
-      }
-
-      /* we hope the last tag is a scriptdataobject containing an index
-       * the size of the last tag is given in the last guint32 bits
-       * then we seek to the beginning of the tag, parse it and hopefully obtain an index */
-      seek_offset = demux->file_size - sizeof (guint32);
-      GST_DEBUG_OBJECT (demux,
-          "File size obtained, seeking to %" G_GUINT64_FORMAT, seek_offset);
-    }
-    GST_OBJECT_UNLOCK (demux);
-
-    if (!building_index) {
-      GST_INFO_OBJECT (demux, "Seeking to last 4 bytes at %" G_GUINT64_FORMAT,
-          seek_offset);
-      return flv_demux_seek_to_offset (demux, seek_offset);
-    }
-
-    /* FIXME: we have to always return true so that we don't block the seek
-     * thread.
-     * Note: maybe it is OK to return true if we're still building the index */
-    return TRUE;
-  }
-
-  return flv_demux_handle_seek_push (demux, event);
-}
-
 static gboolean
 gst_flv_demux_handle_seek_pull (GstFLVDemux * demux, GstEvent * event)
 {
@@ -941,7 +755,6 @@ gst_flv_demux_handle_seek_pull (GstFLVDemux * demux, GstEvent * event)
     goto wrong_format;
 
   flush = !!(flags & GST_SEEK_FLAG_FLUSH);
-  /* FIXME : the keyframe flag is never used */
   keyframe = !!(flags & GST_SEEK_FLAG_KEY_UNIT);
 
   if (flush) {
@@ -1138,11 +951,7 @@ gst_flv_demux_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (demux, "committing index");
         gst_index_commit (demux->index, demux->index_id);
       }
-      if (!demux->no_more_pads) {
-        gst_element_no_more_pads (GST_ELEMENT (demux));
-        demux->no_more_pads = TRUE;
-      }
-
+      gst_element_no_more_pads (GST_ELEMENT (demux));
       if (!gst_flv_demux_push_src_event (demux, event))
         GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
       ret = TRUE;
@@ -1268,32 +1077,6 @@ gst_flv_demux_query (GstPad * pad, GstQuery * query)
       break;
     }
 
-    case GST_QUERY_SEEKING:{
-      GstFormat fmt;
-
-      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
-      res = TRUE;
-      if (fmt != GST_FORMAT_TIME || !demux->index) {
-        gst_query_set_seeking (query, fmt, FALSE, -1, -1);
-      } else if (demux->random_access) {
-        gst_query_set_seeking (query, GST_FORMAT_TIME, TRUE, 0,
-            demux->duration);
-      } else {
-        GstQuery *peerquery = gst_query_new_seeking (GST_FORMAT_BYTES);
-        gboolean seekable = gst_pad_peer_query (demux->sinkpad, peerquery);
-
-        if (seekable)
-          gst_query_parse_seeking (peerquery, NULL, &seekable, NULL, NULL);
-        gst_query_unref (peerquery);
-
-        if (seekable)
-          gst_query_set_seeking (query, GST_FORMAT_TIME, seekable, 0,
-              demux->duration);
-        else
-          gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
-      }
-      break;
-    }
     case GST_QUERY_LATENCY:
     default:
     {
@@ -1374,14 +1157,11 @@ gst_flv_demux_set_index (GstElement * element, GstIndex * index)
   GST_OBJECT_LOCK (demux);
   if (demux->index)
     gst_object_unref (demux->index);
-  if (index) {
-    demux->index = gst_object_ref (index);
-    gst_index_get_writer_id (index, GST_OBJECT (element), &demux->index_id);
-    demux->own_index = FALSE;
-  } else
-    demux->index = NULL;
-
+  demux->index = gst_object_ref (index);
   GST_OBJECT_UNLOCK (demux);
+
+  gst_index_get_writer_id (index, GST_OBJECT (element), &demux->index_id);
+  demux->own_index = FALSE;
 }
 
 static GstIndex *
