@@ -24,23 +24,38 @@
 #endif
 
 #include <string.h>
+#include <gst/floatcast/floatcast.h>
 
 #include "ebml-read.h"
 #include "ebml-ids.h"
 
+#include <math.h>
+
+/* NAN is supposed to be in math.h, Microsoft defines it in xmath.h */
+#ifdef _MSC_VER
+#include <xmath.h>
+#endif
+
+/* If everything goes wrong try 0.0/0.0 which should be NAN */
+#ifndef NAN
+#define NAN (0.0 / 0.0)
+#endif
+
 GST_DEBUG_CATEGORY_STATIC (ebmlread_debug);
 #define GST_CAT_DEFAULT ebmlread_debug
 
 static void gst_ebml_read_class_init (GstEbmlReadClass * klass);
+
 static void gst_ebml_read_init (GstEbmlRead * ebml);
+
 static GstStateChangeReturn gst_ebml_read_change_state (GstElement * element,
     GstStateChange transition);
 
 /* convenience functions */
-static gboolean gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size,
-    GstBuffer ** p_buf);
-static gboolean gst_ebml_read_pull_bytes (GstEbmlRead * ebml, guint size,
-    GstBuffer ** p_buf);
+static GstFlowReturn gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size,
+    GstBuffer ** p_buf, guint8 ** bytes);
+static GstFlowReturn gst_ebml_read_pull_bytes (GstEbmlRead * ebml, guint size,
+    GstBuffer ** p_buf, guint8 ** bytes);
 
 
 static GstElementClass *parent_class;   /* NULL */
@@ -71,16 +86,41 @@ gst_ebml_read_get_type (void)
   return gst_ebml_read_type;
 }
 
+void
+gst_ebml_level_free (GstEbmlLevel * level)
+{
+  g_slice_free (GstEbmlLevel, level);
+}
+
+static void
+gst_ebml_finalize (GObject * obj)
+{
+  GstEbmlRead *ebml = GST_EBML_READ (obj);
+
+  g_list_foreach (ebml->level, (GFunc) gst_ebml_level_free, NULL);
+  g_list_free (ebml->level);
+  ebml->level = NULL;
+  if (ebml->cached_buffer) {
+    gst_buffer_unref (ebml->cached_buffer);
+    ebml->cached_buffer = NULL;
+  }
+
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+
 static void
 gst_ebml_read_class_init (GstEbmlReadClass * klass)
 {
   GstElementClass *gstelement_class = (GstElementClass *) klass;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
   GST_DEBUG_CATEGORY_INIT (ebmlread_debug, "ebmlread",
       0, "EBML stream helper class");
 
+  gobject_class->finalize = gst_ebml_finalize;
+
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_ebml_read_change_state);
 }
@@ -113,7 +153,7 @@ gst_ebml_read_change_state (GstElement * element, GstStateChange transition)
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
     {
-      g_list_foreach (ebml->level, (GFunc) g_free, NULL);
+      g_list_foreach (ebml->level, (GFunc) gst_ebml_level_free, NULL);
       g_list_free (ebml->level);
       ebml->level = NULL;
       if (ebml->cached_buffer) {
@@ -144,12 +184,11 @@ gst_ebml_read_element_level_up (GstEbmlRead * ebml)
   guint64 pos = ebml->offset;
 
   while (ebml->level != NULL) {
-    GList *last = g_list_last (ebml->level);
-    GstEbmlLevel *level = last->data;
+    GstEbmlLevel *level = ebml->level->data;
 
     if (pos >= level->start + level->length) {
-      ebml->level = g_list_remove (ebml->level, level);
-      g_free (level);
+      ebml->level = g_list_delete_link (ebml->level, ebml->level);
+      gst_ebml_level_free (level);
       num++;
     } else {
       break;
@@ -162,8 +201,9 @@ gst_ebml_read_element_level_up (GstEbmlRead * ebml)
 /*
  * Calls pull_range for (offset,size) without advancing our offset
  */
-static gboolean
-gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf)
+static GstFlowReturn
+gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf,
+    guint8 ** bytes)
 {
   GstFlowReturn ret;
 
@@ -175,50 +215,89 @@ gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf)
 
     if (cache_offset <= ebml->offset &&
         (ebml->offset + size) < (cache_offset + cache_size)) {
-      *p_buf = gst_buffer_create_sub (ebml->cached_buffer,
-          ebml->offset - cache_offset, size);
-      return TRUE;
+      if (p_buf)
+        *p_buf = gst_buffer_create_sub (ebml->cached_buffer,
+            ebml->offset - cache_offset, size);
+      if (bytes)
+        *bytes =
+            GST_BUFFER_DATA (ebml->cached_buffer) + ebml->offset - cache_offset;
+      return GST_FLOW_OK;
     }
+    /* not enough data in the cache, free cache and get a new one */
     gst_buffer_unref (ebml->cached_buffer);
     ebml->cached_buffer = NULL;
   }
 
-  if (gst_pad_pull_range (ebml->sinkpad, ebml->offset, MAX (size, 64 * 1024),
-          &ebml->cached_buffer) == GST_FLOW_OK &&
-      GST_BUFFER_SIZE (ebml->cached_buffer) >= size) {
-    *p_buf = gst_buffer_create_sub (ebml->cached_buffer, 0, size);
-    return TRUE;
-  }
-
-  ret = gst_pad_pull_range (ebml->sinkpad, ebml->offset, size, p_buf);
+  /* refill the cache */
+  ret = gst_pad_pull_range (ebml->sinkpad, ebml->offset, MAX (size, 64 * 1024),
+      &ebml->cached_buffer);
   if (ret != GST_FLOW_OK) {
-    GST_DEBUG ("pull_range returned %d", ret);
-    return FALSE;
+    ebml->cached_buffer = NULL;
+    return ret;
   }
 
-  if (GST_BUFFER_SIZE (*p_buf) < size) {
+  if (GST_BUFFER_SIZE (ebml->cached_buffer) >= size) {
+    if (p_buf)
+      *p_buf = gst_buffer_create_sub (ebml->cached_buffer, 0, size);
+    if (bytes)
+      *bytes = GST_BUFFER_DATA (ebml->cached_buffer);
+    return GST_FLOW_OK;
+  }
+
+  /* Not possible to get enough data, try a last time with
+   * requesting exactly the size we need */
+  gst_buffer_unref (ebml->cached_buffer);
+  ebml->cached_buffer = NULL;
+
+  ret =
+      gst_pad_pull_range (ebml->sinkpad, ebml->offset, size,
+      &ebml->cached_buffer);
+  if (ret != GST_FLOW_OK) {
+    GST_DEBUG_OBJECT (ebml, "pull_range returned %d", ret);
+    if (p_buf)
+      *p_buf = NULL;
+    if (bytes)
+      *bytes = NULL;
+    return ret;
+  }
+
+  if (GST_BUFFER_SIZE (ebml->cached_buffer) < size) {
     GST_WARNING_OBJECT (ebml, "Dropping short buffer at offset %"
         G_GUINT64_FORMAT ": wanted %u bytes, got %u bytes", ebml->offset,
-        size, GST_BUFFER_SIZE (*p_buf));
-    gst_buffer_unref (*p_buf);
-    *p_buf = NULL;
-    return FALSE;
+        size, GST_BUFFER_SIZE (ebml->cached_buffer));
+
+    gst_buffer_unref (ebml->cached_buffer);
+    ebml->cached_buffer = NULL;
+    if (p_buf)
+      *p_buf = NULL;
+    if (bytes)
+      *bytes = NULL;
+    return GST_FLOW_UNEXPECTED;
   }
 
-  return TRUE;
+  if (p_buf)
+    *p_buf = gst_buffer_create_sub (ebml->cached_buffer, 0, size);
+  if (bytes)
+    *bytes = GST_BUFFER_DATA (*p_buf);
+
+  return GST_FLOW_OK;
 }
 
 /*
  * Calls pull_range for (offset,size) and advances our offset by size
  */
-static gboolean
-gst_ebml_read_pull_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf)
+static GstFlowReturn
+gst_ebml_read_pull_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf,
+    guint8 ** bytes)
 {
-  if (!gst_ebml_read_peek_bytes (ebml, size, p_buf))
-    return FALSE;
+  GstFlowReturn ret;
+
+  ret = gst_ebml_read_peek_bytes (ebml, size, p_buf, bytes);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   ebml->offset += size;
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 /*
@@ -226,19 +305,20 @@ gst_ebml_read_pull_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf)
  * Return: FALSE on error.
  */
 
-static gboolean
+static GstFlowReturn
 gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
 {
-  GstBuffer *buf;
+  guint8 *buf;
   gint len_mask = 0x80, read = 1, n = 1;
   guint32 total;
   guint8 b;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_peek_bytes (ebml, 1, &buf))
-    return FALSE;
+  ret = gst_ebml_read_peek_bytes (ebml, 1, NULL, &buf);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  b = GST_READ_UINT8 (GST_BUFFER_DATA (buf));
-  gst_buffer_unref (buf);
+  b = GST_READ_UINT8 (buf);
 
   total = (guint32) b;
 
@@ -247,19 +327,18 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
     len_mask >>= 1;
   }
   if (read > 4) {
-    guint64 pos = ebml->offset;
-
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid EBML ID size tag (0x%x) at position %llu (0x%llx)",
-            (guint) b, pos, pos));
-    return FALSE;
+    GST_ERROR_OBJECT (ebml,
+        "Invalid EBML ID size tag (0x%x) at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", (guint) b, ebml->offset, ebml->offset);
+    return GST_FLOW_ERROR;
   }
 
-  if (!gst_ebml_read_peek_bytes (ebml, read, &buf))
-    return FALSE;
+  ret = gst_ebml_read_peek_bytes (ebml, read, NULL, &buf);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   while (n < read) {
-    b = GST_READ_UINT8 (GST_BUFFER_DATA (buf) + n);
+    b = GST_READ_UINT8 (buf + n);
     total = (total << 8) | b;
     ++n;
   }
@@ -270,10 +349,8 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
   if (level_up)
     *level_up = gst_ebml_read_element_level_up (ebml);
 
-  gst_buffer_unref (buf);
-
   ebml->offset += read;
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 /*
@@ -281,19 +358,21 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
  * Return: the number of bytes read or -1 on error.
  */
 
-static gint
-gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
+static GstFlowReturn
+gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length,
+    gint * rread)
 {
-  GstBuffer *buf;
+  GstFlowReturn ret;
+  guint8 *buf;
   gint len_mask = 0x80, read = 1, n = 1, num_ffs = 0;
   guint64 total;
   guint8 b;
 
-  if (!gst_ebml_read_peek_bytes (ebml, 1, &buf))
-    return -1;
+  ret = gst_ebml_read_peek_bytes (ebml, 1, NULL, &buf);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  b = GST_READ_UINT8 (GST_BUFFER_DATA (buf));
-  gst_buffer_unref (buf);
+  b = GST_READ_UINT8 (buf);
 
   total = (guint64) b;
 
@@ -302,38 +381,39 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
     len_mask >>= 1;
   }
   if (read > 8) {
-    guint64 pos = ebml->offset;
-
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid EBML length size tag (0x%x) at position %llu (0x%llx)",
-            (guint) b, pos, pos));
-    return -1;
+    GST_ERROR_OBJECT (ebml,
+        "Invalid EBML length size tag (0x%x) at position %" G_GUINT64_FORMAT
+        " (0x%" G_GINT64_MODIFIER "x)", (guint) b, ebml->offset, ebml->offset);
+    return GST_FLOW_ERROR;
   }
 
   if ((total &= (len_mask - 1)) == len_mask - 1)
     num_ffs++;
 
-  if (!gst_ebml_read_peek_bytes (ebml, read, &buf))
-    return -1;
+  ret = gst_ebml_read_peek_bytes (ebml, read, NULL, &buf);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   while (n < read) {
-    guint8 b = GST_READ_UINT8 (GST_BUFFER_DATA (buf) + n);
+    guint8 b = GST_READ_UINT8 (buf + n);
 
     if (b == 0xff)
       num_ffs++;
     total = (total << 8) | b;
     ++n;
   }
-  gst_buffer_unref (buf);
 
   if (read == num_ffs)
     *length = G_MAXUINT64;
   else
     *length = total;
 
+  if (rread)
+    *rread = read;
+
   ebml->offset += read;
 
-  return read;
+  return GST_FLOW_OK;
 }
 
 /*
@@ -342,20 +422,45 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
  * next element lies higher than the previous one.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_peek_id (GstEbmlRead * ebml, guint * level_up, guint32 * id)
 {
   guint64 off;
+  guint level_up_tmp = 0;
+  GstFlowReturn ret;
 
   g_assert (level_up);
+  g_assert (id);
 
+  *level_up = 0;
+
+next:
   off = ebml->offset;           /* save offset */
 
-  if (!gst_ebml_read_element_id (ebml, id, level_up))
-    return FALSE;
+  if ((ret = gst_ebml_read_element_id (ebml, id, &level_up_tmp)) != GST_FLOW_OK)
+    return ret;
 
   ebml->offset = off;           /* restore offset */
-  return TRUE;
+
+  *level_up += level_up_tmp;
+  level_up_tmp = 0;
+
+  switch (*id) {
+    case GST_EBML_ID_VOID:
+      GST_DEBUG_OBJECT (ebml, "Skipping EBML Void element");
+      if ((ret = gst_ebml_read_skip (ebml)) != GST_FLOW_OK)
+        return ret;
+      goto next;
+      break;
+    case GST_EBML_ID_CRC32:
+      GST_DEBUG_OBJECT (ebml, "Skipping EBML CRC32 element");
+      if ((ret = gst_ebml_read_skip (ebml)) != GST_FLOW_OK)
+        return ret;
+      goto next;
+      break;
+  }
+
+  return ret;
 }
 
 /*
@@ -368,11 +473,10 @@ gst_ebml_read_get_length (GstEbmlRead * ebml)
   GstFormat fmt = GST_FORMAT_BYTES;
   gint64 end;
 
-  if (!gst_pad_query_duration (GST_PAD_PEER (ebml->sinkpad), &fmt, &end))
-    g_return_val_if_reached (0);        ///// FIXME /////////
-
-  if (fmt != GST_FORMAT_BYTES || end < 0)
-    g_return_val_if_reached (0);        ///// FIXME /////////
+  /* FIXME: what to do if we don't get the upstream length */
+  if (!gst_pad_query_peer_duration (ebml->sinkpad, &fmt, &end) ||
+      fmt != GST_FORMAT_BYTES || end < 0)
+    g_return_val_if_reached (0);
 
   return end;
 }
@@ -381,133 +485,171 @@ gst_ebml_read_get_length (GstEbmlRead * ebml)
  * Seek to a given offset.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_seek (GstEbmlRead * ebml, guint64 offset)
 {
   if (offset >= gst_ebml_read_get_length (ebml))
-    return FALSE;
+    return GST_FLOW_UNEXPECTED;
 
   ebml->offset = offset;
 
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 /*
  * Skip the next element.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_skip (GstEbmlRead * ebml)
 {
   guint64 length;
   guint32 id;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_element_id (ebml, &id, NULL))
-    return FALSE;
+  ret = gst_ebml_read_element_id (ebml, &id, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  if (gst_ebml_read_element_length (ebml, &length) < 0)
-    return FALSE;
+  ret = gst_ebml_read_element_length (ebml, &length, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   ebml->offset += length;
-  return TRUE;
+  return ret;
 }
 
 /*
  * Read the next element as a GstBuffer (binary).
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_buffer (GstEbmlRead * ebml, guint32 * id, GstBuffer ** buf)
 {
   guint64 length;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_element_id (ebml, id, NULL))
-    return FALSE;
+  ret = gst_ebml_read_element_id (ebml, id, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  if (gst_ebml_read_element_length (ebml, &length) < 0)
-    return FALSE;
+  ret = gst_ebml_read_element_length (ebml, &length, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   if (length == 0) {
     *buf = gst_buffer_new ();
-    return TRUE;
+    return GST_FLOW_OK;
   }
 
   *buf = NULL;
-  if (!gst_ebml_read_pull_bytes (ebml, (guint) length, buf))
-    return FALSE;
+  ret = gst_ebml_read_pull_bytes (ebml, (guint) length, buf, NULL);
 
-  return TRUE;
+  return ret;
+}
+
+/*
+ * Read the next element, return a pointer to it and its size.
+ */
+
+static GstFlowReturn
+gst_ebml_read_bytes (GstEbmlRead * ebml, guint32 * id, guint8 ** data,
+    guint * size)
+{
+  guint64 length;
+  GstFlowReturn ret;
+
+  *size = 0;
+
+  ret = gst_ebml_read_element_id (ebml, id, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  ret = gst_ebml_read_element_length (ebml, &length, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  if (length == 0) {
+    *data = NULL;
+    return ret;
+  }
+
+  *data = NULL;
+  ret = gst_ebml_read_pull_bytes (ebml, (guint) length, NULL, data);
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  *size = (guint) length;
+
+  return ret;
 }
 
 /*
  * Read the next element as an unsigned int.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_uint (GstEbmlRead * ebml, guint32 * id, guint64 * num)
 {
-  GstBuffer *buf;
   guint8 *data;
   guint size;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_buffer (ebml, id, &buf))
-    return FALSE;
+  ret = gst_ebml_read_bytes (ebml, id, &data, &size);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid integer element size %d at position %llu (0x%llu)",
-            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
-    gst_buffer_unref (buf);
-    return FALSE;
+    GST_ERROR_OBJECT (ebml,
+        "Invalid integer element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
+    return GST_FLOW_ERROR;
   }
   *num = 0;
   while (size > 0) {
-    *num = (*num << 8) | data[GST_BUFFER_SIZE (buf) - size];
+    *num = (*num << 8) | *data;
     size--;
+    data++;
   }
 
-  gst_buffer_unref (buf);
-
-  return TRUE;
+  return ret;
 }
 
 /*
  * Read the next element as a signed int.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_sint (GstEbmlRead * ebml, guint32 * id, gint64 * num)
 {
-  GstBuffer *buf;
   guint8 *data;
-  guint size, negative = 0, n = 0;
+  guint size;
+  gboolean negative = 0;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_buffer (ebml, id, &buf))
-    return FALSE;
+  ret = gst_ebml_read_bytes (ebml, id, &data, &size);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid integer element size %d at position %llu (0x%llx)",
-            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
-    gst_buffer_unref (buf);
-    return FALSE;
-  }
-
-  buf = gst_buffer_make_writable (buf);
-
-  data = GST_BUFFER_DATA (buf);
-
-  if (data[0] & 0x80) {
-    negative = 1;
-    data[0] &= ~0x80;
+    GST_ERROR_OBJECT (ebml,
+        "Invalid integer element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
+    return GST_FLOW_ERROR;
   }
 
   *num = 0;
-  while (n < size) {
-    *num = (*num << 8) | data[n++];
+  if (*data & 0x80) {
+    negative = 1;
+    *num = *data & ~0x80;
+    size--;
+    data++;
+  }
+
+  while (size > 0) {
+    *num = (*num << 8) | *data;
+    size--;
+    data++;
   }
 
   /* make signed */
@@ -515,115 +657,128 @@ gst_ebml_read_sint (GstEbmlRead * ebml, guint32 * id, gint64 * num)
     *num = 0 - *num;
   }
 
-  gst_buffer_unref (buf);
+  return ret;
+}
 
-  return TRUE;
+/* Convert 80 bit extended precision float in big endian format to double.
+ * Code taken from libavutil/intfloat_readwrite.c from ffmpeg,
+ * licensed under LGPL */
+
+struct _ext_float
+{
+  guint8 exponent[2];
+  guint8 mantissa[8];
+};
+
+static gdouble
+_ext2dbl (guint8 * data)
+{
+  struct _ext_float ext;
+  guint64 m = 0;
+  gint e, i;
+
+  memcpy (&ext.exponent, data, 2);
+  memcpy (&ext.mantissa, data + 2, 8);
+
+  for (i = 0; i < 8; i++)
+    m = (m << 8) + ext.mantissa[i];
+  e = (((gint) ext.exponent[0] & 0x7f) << 8) | ext.exponent[1];
+  if (e == 0x7fff && m)
+    return NAN;
+  e -= 16383 + 63;              /* In IEEE 80 bits, the whole (i.e. 1.xxxx)
+                                 * mantissa bit is written as opposed to the
+                                 * single and double precision formats */
+  if (ext.exponent[0] & 0x80)
+    m = -m;
+  return ldexp (m, e);
 }
 
 /*
  * Read the next element as a float.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_float (GstEbmlRead * ebml, guint32 * id, gdouble * num)
 {
-  GstBuffer *buf;
   guint8 *data;
   guint size;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_buffer (ebml, id, &buf))
-    return FALSE;
-
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
+  ret = gst_ebml_read_bytes (ebml, id, &data, &size);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   if (size != 4 && size != 8 && size != 10) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid float element size %d at position %llu (0x%llx)",
-            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
-    gst_buffer_unref (buf);
-    return FALSE;
-  }
-
-  if (size == 10) {
-    GST_ELEMENT_ERROR (ebml, CORE, NOT_IMPLEMENTED, (NULL),
-        ("FIXME! 10-byte floats unimplemented"));
-    gst_buffer_unref (buf);
-    return FALSE;
+    GST_ERROR_OBJECT (ebml,
+        "Invalid float element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
+    return GST_FLOW_ERROR;
   }
 
   if (size == 4) {
     gfloat f;
 
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-    f = *(gfloat *) data;
-#else
-    while (size > 0) {
-      ((guint8 *) & f)[size - 1] = data[4 - size];
-      size--;
-    }
-#endif
+    memcpy (&f, data, 4);
+    f = GFLOAT_FROM_BE (f);
 
     *num = f;
-  } else {
+  } else if (size == 8) {
     gdouble d;
 
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-    d = *(gdouble *) data;
-#else
-    while (size > 0) {
-      ((guint8 *) & d)[size - 1] = data[8 - size];
-      size--;
-    }
-#endif
+    memcpy (&d, data, 8);
+    d = GDOUBLE_FROM_BE (d);
 
     *num = d;
+  } else {
+    *num = _ext2dbl (data);
   }
 
-  gst_buffer_unref (buf);
-
-  return TRUE;
+  return ret;
 }
 
 /*
  * Read the next element as an ASCII string.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_ascii (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 {
-  GstBuffer *buf;
+  guint8 *data;
+  guint size;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_buffer (ebml, id, &buf))
-    return FALSE;
+  ret = gst_ebml_read_bytes (ebml, id, &data, &size);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  *str = g_malloc (GST_BUFFER_SIZE (buf) + 1);
-  memcpy (*str, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
-  (*str)[GST_BUFFER_SIZE (buf)] = '\0';
+  *str = g_malloc (size + 1);
+  memcpy (*str, data, size);
+  (*str)[size] = '\0';
 
-  gst_buffer_unref (buf);
-
-  return TRUE;
+  return ret;
 }
 
 /*
  * Read the next element as a UTF-8 string.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_utf8 (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 {
-  gboolean ret;
+  GstFlowReturn ret;
 
 #ifndef GST_DISABLE_GST_DEBUG
   guint64 oldoff = ebml->offset;
 #endif
 
   ret = gst_ebml_read_ascii (ebml, id, str);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   if (str != NULL && *str != NULL && **str != '\0' &&
       !g_utf8_validate (*str, -1, NULL)) {
-    GST_WARNING ("Invalid UTF-8 string at offset %" G_GUINT64_FORMAT, oldoff);
+    GST_WARNING_OBJECT (ebml,
+        "Invalid UTF-8 string at offset %" G_GUINT64_FORMAT, oldoff);
   }
 
   return ret;
@@ -634,14 +789,19 @@ gst_ebml_read_utf8 (GstEbmlRead * ebml, guint32 * id, gchar ** str)
  * Returns the seconds since the unix epoch.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_date (GstEbmlRead * ebml, guint32 * id, gint64 * date)
 {
   gint64 ebml_date;
-  gboolean res = gst_ebml_read_sint (ebml, id, &ebml_date);
+  GstFlowReturn ret;
+
+  ret = gst_ebml_read_sint (ebml, id, &ebml_date);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   *date = (ebml_date / GST_SECOND) + GST_EBML_DATE_OFFSET;
-  return res;
+
+  return ret;
 }
 
 /*
@@ -649,58 +809,63 @@ gst_ebml_read_date (GstEbmlRead * ebml, guint32 * id, gint64 * date)
  * are supposed to be sub-elements which can be read separately.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_master (GstEbmlRead * ebml, guint32 * id)
 {
   GstEbmlLevel *level;
   guint64 length;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_element_id (ebml, id, NULL))
-    return FALSE;
+  ret = gst_ebml_read_element_id (ebml, id, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  if (gst_ebml_read_element_length (ebml, &length) < 0)
-    return FALSE;
+  ret = gst_ebml_read_element_length (ebml, &length, NULL);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   /* remember level */
-  level = g_new (GstEbmlLevel, 1);
+  level = g_slice_new (GstEbmlLevel);
   level->start = ebml->offset;
   level->length = length;
-  ebml->level = g_list_append (ebml->level, level);
+  ebml->level = g_list_prepend (ebml->level, level);
 
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 /*
  * Read the next element as binary data.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_binary (GstEbmlRead * ebml,
     guint32 * id, guint8 ** binary, guint64 * length)
 {
-  GstBuffer *buf;
+  guint8 *data;
+  guint size;
+  GstFlowReturn ret;
 
-  if (!gst_ebml_read_buffer (ebml, id, &buf))
-    return FALSE;
+  ret = gst_ebml_read_bytes (ebml, id, &data, &size);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
-  *length = GST_BUFFER_SIZE (buf);
-  *binary = g_memdup (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+  *length = size;
+  *binary = g_memdup (data, size);
 
-  gst_buffer_unref (buf);
-
-  return TRUE;
+  return GST_FLOW_OK;
 }
 
 /*
  * Read an EBML header.
  */
 
-gboolean
+GstFlowReturn
 gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
 {
   /* this function is the first to be called */
   guint32 id;
   guint level_up;
+  GstFlowReturn ret;
 
   /* default init */
   if (doctype)
@@ -708,21 +873,24 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
   if (version)
     *version = 1;
 
-  if (!gst_ebml_peek_id (ebml, &level_up, &id))
-    return FALSE;
+  ret = gst_ebml_peek_id (ebml, &level_up, &id);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   GST_DEBUG_OBJECT (ebml, "id: %08x", GST_READ_UINT32_BE (&id));
 
   if (level_up != 0 || id != GST_EBML_ID_HEADER) {
-    GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
-    return FALSE;
+    GST_ERROR_OBJECT (ebml, "Failed to read header");
+    return GST_FLOW_ERROR;
   }
-  if (!gst_ebml_read_master (ebml, &id))
-    return FALSE;
+  ret = gst_ebml_read_master (ebml, &id);
+  if (ret != GST_FLOW_OK)
+    return ret;
 
   while (TRUE) {
-    if (!gst_ebml_peek_id (ebml, &level_up, &id))
-      return FALSE;
+    ret = gst_ebml_peek_id (ebml, &level_up, &id);
+    if (ret != GST_FLOW_OK)
+      return ret;
 
     /* end-of-header */
     if (level_up)
@@ -733,11 +901,17 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
       case GST_EBML_ID_EBMLREADVERSION:{
         guint64 num;
 
-        if (!gst_ebml_read_uint (ebml, &id, &num))
-          return FALSE;
+        ret = gst_ebml_read_uint (ebml, &id, &num);
+        if (ret != GST_FLOW_OK)
+          return ret;
         g_assert (id == GST_EBML_ID_EBMLREADVERSION);
-        if (num != GST_EBML_VERSION)
-          return FALSE;
+        if (num != GST_EBML_VERSION) {
+          GST_ERROR_OBJECT (ebml, "Unsupported EBML version %" G_GUINT64_FORMAT,
+              num);
+          return GST_FLOW_ERROR;
+        }
+
+        GST_DEBUG_OBJECT (ebml, "EbmlReadVersion: %" G_GUINT64_FORMAT, num);
         break;
       }
 
@@ -745,11 +919,16 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
       case GST_EBML_ID_EBMLMAXSIZELENGTH:{
         guint64 num;
 
-        if (!gst_ebml_read_uint (ebml, &id, &num))
-          return FALSE;
+        ret = gst_ebml_read_uint (ebml, &id, &num);
+        if (ret != GST_FLOW_OK)
+          return ret;
         g_assert (id == GST_EBML_ID_EBMLMAXSIZELENGTH);
-        if (num != sizeof (guint64))
-          return FALSE;
+        if (num > sizeof (guint64)) {
+          GST_ERROR_OBJECT (ebml,
+              "Unsupported EBML maximum size %" G_GUINT64_FORMAT, num);
+          return GST_FLOW_ERROR;
+        }
+        GST_DEBUG_OBJECT (ebml, "EbmlMaxSizeLength: %" G_GUINT64_FORMAT, num);
         break;
       }
 
@@ -757,23 +936,31 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
       case GST_EBML_ID_EBMLMAXIDLENGTH:{
         guint64 num;
 
-        if (!gst_ebml_read_uint (ebml, &id, &num))
-          return FALSE;
+        ret = gst_ebml_read_uint (ebml, &id, &num);
+        if (ret != GST_FLOW_OK)
+          return ret;
         g_assert (id == GST_EBML_ID_EBMLMAXIDLENGTH);
-        if (num != sizeof (guint32))
-          return FALSE;
+        if (num > sizeof (guint32)) {
+          GST_ERROR_OBJECT (ebml,
+              "Unsupported EBML maximum ID %" G_GUINT64_FORMAT, num);
+          return GST_FLOW_ERROR;
+        }
+        GST_DEBUG_OBJECT (ebml, "EbmlMaxIdLength: %" G_GUINT64_FORMAT, num);
         break;
       }
 
       case GST_EBML_ID_DOCTYPE:{
         gchar *text;
 
-        if (!gst_ebml_read_ascii (ebml, &id, &text))
-          return FALSE;
+        ret = gst_ebml_read_ascii (ebml, &id, &text);
+        if (ret != GST_FLOW_OK)
+          return ret;
         g_assert (id == GST_EBML_ID_DOCTYPE);
+
+        GST_DEBUG_OBJECT (ebml, "EbmlDocType: %s", GST_STR_NULL (text));
+
         if (doctype) {
-          if (doctype)
-            g_free (*doctype);
+          g_free (*doctype);
           *doctype = text;
         } else
           g_free (text);
@@ -783,27 +970,30 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
       case GST_EBML_ID_DOCTYPEREADVERSION:{
         guint64 num;
 
-        if (!gst_ebml_read_uint (ebml, &id, &num))
-          return FALSE;
+        ret = gst_ebml_read_uint (ebml, &id, &num);
+        if (ret != GST_FLOW_OK)
+          return ret;
         g_assert (id == GST_EBML_ID_DOCTYPEREADVERSION);
         if (version)
           *version = num;
+        GST_DEBUG_OBJECT (ebml, "EbmlReadVersion: %" G_GUINT64_FORMAT, num);
         break;
       }
 
       default:
-        GST_WARNING ("Unknown data type 0x%x in EBML header (ignored)", id);
+        GST_WARNING_OBJECT (ebml,
+            "Unknown data type 0x%x in EBML header (ignored)", id);
         /* pass-through */
 
         /* we ignore these two, as they don't tell us anything we care about */
-      case GST_EBML_ID_VOID:
       case GST_EBML_ID_EBMLVERSION:
       case GST_EBML_ID_DOCTYPEVERSION:
-        if (!gst_ebml_read_skip (ebml))
-          return FALSE;
+        ret = gst_ebml_read_skip (ebml);
+        if (ret != GST_FLOW_OK)
+          return ret;
         break;
     }
   }
 
-  return TRUE;
+  return GST_FLOW_OK;
 }
