@@ -25,6 +25,7 @@
 #endif
 
 #include <string.h>
+#include <gst/floatcast/floatcast.h>
 
 #include "ebml-write.h"
 #include "ebml-ids.h"
@@ -71,6 +72,11 @@ gst_ebml_write_finalize (GObject * object)
 
   gst_object_unref (ebml->srcpad);
 
+  if (ebml->cache) {
+    gst_buffer_unref (ebml->cache);
+    ebml->cache = NULL;
+  }
+
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -153,7 +159,7 @@ gst_ebml_last_write_result (GstEbmlWrite * ebml)
 void
 gst_ebml_write_set_cache (GstEbmlWrite * ebml, guint size)
 {
-  /* This is currently broken. I don't know why yet. */
+  /* FIXME: This is currently broken. I don't know why yet. */
   return;
 
   g_return_if_fail (ebml->cache == NULL);
@@ -532,21 +538,12 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
 void
 gst_ebml_write_float (GstEbmlWrite * ebml, guint32 id, gdouble num)
 {
-#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
-  gint n;
-#endif
   GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
 
   gst_ebml_write_element_id (buf, id);
   gst_ebml_write_element_size (buf, 8);
-#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
-  for (n = 0; n < 8; n++) {
-    GST_BUFFER_DATA (buf)[GST_BUFFER_SIZE (buf)] = ((guint8 *) & num)[7 - n];
-    GST_BUFFER_SIZE (buf) += 1;
-  }
-#else
+  num = GDOUBLE_TO_BE (num);
   gst_ebml_write_element_data (buf, (guint8 *) & num, 8);
-#endif
   gst_ebml_write_element_push (ebml, buf);
 }
 
