@@ -37,6 +37,13 @@ GST_DEBUG_CATEGORY_STATIC (rtptheorapay_debug);
  * http://svn.xiph.org/trunk/theora/doc/draft-ietf-avt-rtp-theora-01.txt
  */
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_theorapay_details =
+GST_ELEMENT_DETAILS ("RTP Theora payloader",
+    "Codec/Payloader/Network",
+    "Payload-encode Theora video into RTP packets (draft-01 RFC XXXX)",
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 static GstStaticPadTemplate gst_rtp_theora_pay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -44,18 +51,19 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\""
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\", "
+        "delivery-method = (string) \"inline\""
         /* All required parameters
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
          * "width = (string) [1, 1048561] (multiples of 16) "
          * "height = (string) [1, 1048561] (multiples of 16) "
+         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
          * "configuration = (string) ANY"
          */
         /* All optional parameters
          *
          * "configuration-uri ="
-         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
          */
     )
     );
@@ -67,14 +75,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("video/x-theora")
     );
 
-#define DEFAULT_CONFIG_INTERVAL 0
-
-enum
-{
-  PROP_0,
-  PROP_CONFIG_INTERVAL
-};
-
 GST_BOILERPLATE (GstRtpTheoraPay, gst_rtp_theora_pay, GstBaseRTPPayload,
     GST_TYPE_BASE_RTP_PAYLOAD);
 
@@ -84,76 +84,45 @@ static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement *
     element, GstStateChange transition);
 static GstFlowReturn gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
-static gboolean gst_rtp_theora_pay_handle_event (GstPad * pad,
-    GstEvent * event);
-
-
-static void gst_rtp_theora_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_theora_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
 
 static void
 gst_rtp_theora_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_theora_pay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_theora_pay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_theora_pay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_theora_pay_sink_template));
 
-  gst_element_class_set_details_simple (element_class, "RTP Theora payloader",
-      "Codec/Payloader/Network/RTP",
-      "Payload-encode Theora video into RTP packets (draft-01 RFC XXXX)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_theorapay_details);
 }
 
 static void
 gst_rtp_theora_pay_class_init (GstRtpTheoraPayClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gstelement_class->change_state = gst_rtp_theora_pay_change_state;
 
   gstbasertppayload_class->set_caps = gst_rtp_theora_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_theora_pay_handle_buffer;
-  gstbasertppayload_class->handle_event = gst_rtp_theora_pay_handle_event;
-
-  gobject_class->set_property = gst_rtp_theora_pay_set_property;
-  gobject_class->get_property = gst_rtp_theora_pay_get_property;
 
   GST_DEBUG_CATEGORY_INIT (rtptheorapay_debug, "rtptheorapay", 0,
       "Theora RTP Payloader");
-
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CONFIG_INTERVAL,
-      g_param_spec_uint ("config-interval", "Config Send Interval",
-          "Send Config Insertion Interval in seconds (configuration headers "
-          "will be multiplexed in the data stream when detected.) (0 = disabled)",
-          0, 3600, DEFAULT_CONFIG_INTERVAL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
-      );
 }
 
 static void
 gst_rtp_theora_pay_init (GstRtpTheoraPay * rtptheorapay,
     GstRtpTheoraPayClass * klass)
 {
-  rtptheorapay->last_config = GST_CLOCK_TIME_NONE;
-}
-
-static void
-gst_rtp_theora_pay_clear_packet (GstRtpTheoraPay * rtptheorapay)
-{
-  if (rtptheorapay->packet)
-    gst_buffer_unref (rtptheorapay->packet);
-  rtptheorapay->packet = NULL;
+  /* needed because of GST_BOILERPLATE */
 }
 
 static void
@@ -163,12 +132,9 @@ gst_rtp_theora_pay_cleanup (GstRtpTheoraPay * rtptheorapay)
   g_list_free (rtptheorapay->headers);
   rtptheorapay->headers = NULL;
 
-  gst_rtp_theora_pay_clear_packet (rtptheorapay);
-
-  if (rtptheorapay->config_data)
-    g_free (rtptheorapay->config_data);
-  rtptheorapay->config_data = NULL;
-  rtptheorapay->last_config = GST_CLOCK_TIME_NONE;
+  if (rtptheorapay->packet)
+    gst_buffer_unref (rtptheorapay->packet);
+  rtptheorapay->packet = NULL;
 }
 
 static gboolean
@@ -270,7 +236,7 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
   GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
   GList *walk;
-  guint length, size, n_headers, configlen, extralen;
+  guint length, size, n_headers, configlen;
   gchar *wstr, *hstr, *configuration;
   guint8 *data, *config;
   guint32 ident;
@@ -331,7 +297,6 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   length = 0;
   n_headers = 0;
   ident = fnv1_hash_32_new ();
-  extralen = 1;
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
 
@@ -346,7 +311,6 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
     if (g_list_next (walk)) {
       do {
         size++;
-        extralen++;
         bsize >>= 7;
       } while (bsize);
     }
@@ -425,14 +389,6 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 
   /* serialize to base64 */
   configuration = g_base64_encode (config, configlen);
-
-  /* store for later re-sending */
-  rtptheorapay->config_size = configlen - 4 - 3 - 2;
-  rtptheorapay->config_data = g_malloc (rtptheorapay->config_size);
-  rtptheorapay->config_extra_len = extralen;
-  memcpy (rtptheorapay->config_data, config + 4 + 3 + 2,
-      rtptheorapay->config_size);
-
   g_free (config);
 
   /* configure payloader settings */
@@ -520,19 +476,70 @@ invalid_version:
 }
 
 static GstFlowReturn
-gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
-    guint8 * data, guint size, GstClockTime timestamp, GstClockTime duration,
-    guint not_in_length)
+gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
+    GstBuffer * buffer)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
-  guint newsize;
+  GstRtpTheoraPay *rtptheorapay;
+  GstFlowReturn ret;
+  guint size, newsize;
+  guint8 *data;
   guint packet_len;
-  GstClockTime newduration;
+  GstClockTime duration, newduration, timestamp;
   gboolean flush;
+  guint8 TDT;
   guint plen;
   guint8 *ppos, *payload;
   gboolean fragmented;
 
+  rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
+
+  size = GST_BUFFER_SIZE (buffer);
+  data = GST_BUFFER_DATA (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+
+  GST_DEBUG_OBJECT (rtptheorapay, "size %u, duration %" GST_TIME_FORMAT,
+      size, GST_TIME_ARGS (duration));
+
+  if (G_UNLIKELY (size < 1 || size > 0xffff))
+    goto wrong_size;
+
+  /* find packet type */
+  if (data[0] & 0x80) {
+    /* header */
+    if (data[0] == 0x80) {
+      /* identification, we need to parse this in order to get the clock rate.
+       */
+      if (G_UNLIKELY (!gst_rtp_theora_pay_parse_id (basepayload, data, size)))
+        goto parse_id_failed;
+      TDT = 1;
+    } else if (data[0] == 0x81) {
+      /* comment */
+      TDT = 2;
+    } else if (data[0] == 0x82) {
+      /* setup */
+      TDT = 1;
+    } else
+      goto unknown_header;
+  } else
+    /* data */
+    TDT = 0;
+
+  if (rtptheorapay->need_headers) {
+    /* we need to collect the headers and construct a config string from them */
+    if (TDT != 0) {
+      GST_DEBUG_OBJECT (rtptheorapay, "collecting header, buffer %p", buffer);
+      /* append header to the list of headers */
+      rtptheorapay->headers = g_list_append (rtptheorapay->headers, buffer);
+      ret = GST_FLOW_OK;
+      goto done;
+    } else {
+      if (!gst_rtp_theora_pay_finish_headers (basepayload))
+        goto header_error;
+      rtptheorapay->need_headers = FALSE;
+    }
+  }
+
   /* size increases with packet length and 2 bytes size eader. */
   newduration = rtptheorapay->payload_duration;
   if (duration != GST_CLOCK_TIME_NONE)
@@ -542,15 +549,14 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
   packet_len = gst_rtp_buffer_calc_packet_len (newsize, 0, 0);
 
   /* check buffer filled against length and max latency */
-  flush = gst_basertppayload_is_filled (GST_BASE_RTP_PAYLOAD (rtptheorapay),
-      packet_len, newduration);
+  flush = gst_basertppayload_is_filled (basepayload, packet_len, newduration);
   /* we can store up to 15 theora packets in one RTP packet. */
   flush |= (rtptheorapay->payload_pkts == 15);
   /* flush if we have a new TDT */
   if (rtptheorapay->packet)
     flush |= (rtptheorapay->payload_TDT != TDT);
   if (flush)
-    ret = gst_rtp_theora_pay_flush_packet (rtptheorapay);
+    gst_rtp_theora_pay_flush_packet (rtptheorapay);
 
   /* create new packet if we must */
   if (!rtptheorapay->packet) {
@@ -561,22 +567,19 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
   ppos = payload + rtptheorapay->payload_pos;
   fragmented = FALSE;
 
+  ret = GST_FLOW_OK;
+
   /* put buffer in packet, it either fits completely or needs to be fragmented
    * over multiple RTP packets. */
-  do {
+  while (size) {
     plen = MIN (rtptheorapay->payload_left - 2, size);
 
     GST_DEBUG_OBJECT (rtptheorapay, "append %u bytes", plen);
 
     /* data is copied in the payload with a 2 byte length header */
-    ppos[0] = ((plen - not_in_length) >> 8) & 0xff;
-    ppos[1] = ((plen - not_in_length) & 0xff);
-    if (plen)
-      memcpy (&ppos[2], data, plen);
-
-    /* only first (only) configuration cuts length field */
-    /* NOTE: spec (if any) is not clear on this ... */
-    not_in_length = 0;
+    ppos[0] = (plen >> 8) & 0xff;
+    ppos[1] = (plen & 0xff);
+    memcpy (&ppos[2], data, plen);
 
     size -= plen;
     data += plen;
@@ -618,126 +621,7 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
       if (duration != GST_CLOCK_TIME_NONE)
         rtptheorapay->payload_duration += duration;
     }
-  } while (size);
-
-  return ret;
-}
-
-static GstFlowReturn
-gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
-    GstBuffer * buffer)
-{
-  GstRtpTheoraPay *rtptheorapay;
-  GstFlowReturn ret;
-  guint size;
-  guint8 *data;
-  GstClockTime duration, timestamp;
-  guint8 TDT;
-  gboolean keyframe = FALSE;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
-
-  size = GST_BUFFER_SIZE (buffer);
-  data = GST_BUFFER_DATA (buffer);
-  duration = GST_BUFFER_DURATION (buffer);
-  timestamp = GST_BUFFER_TIMESTAMP (buffer);
-
-  GST_DEBUG_OBJECT (rtptheorapay, "size %u, duration %" GST_TIME_FORMAT,
-      size, GST_TIME_ARGS (duration));
-
-  if (G_UNLIKELY (size > 0xffff))
-    goto wrong_size;
-
-  /* find packet type */
-  if (size == 0) {
-    TDT = 0;
-    keyframe = FALSE;
-  } else if (data[0] & 0x80) {
-    /* header */
-    if (data[0] == 0x80) {
-      /* identification, we need to parse this in order to get the clock rate.
-       */
-      if (G_UNLIKELY (!gst_rtp_theora_pay_parse_id (basepayload, data, size)))
-        goto parse_id_failed;
-      TDT = 1;
-    } else if (data[0] == 0x81) {
-      /* comment */
-      TDT = 2;
-    } else if (data[0] == 0x82) {
-      /* setup */
-      TDT = 1;
-    } else
-      goto unknown_header;
-  } else {
-    /* data */
-    TDT = 0;
-    keyframe = ((data[0] & 0x40) == 0);
   }
-
-  if (rtptheorapay->need_headers) {
-    /* we need to collect the headers and construct a config string from them */
-    if (TDT != 0) {
-      GST_DEBUG_OBJECT (rtptheorapay, "collecting header, buffer %p", buffer);
-      /* append header to the list of headers */
-      rtptheorapay->headers = g_list_append (rtptheorapay->headers, buffer);
-      ret = GST_FLOW_OK;
-      goto done;
-    } else {
-      if (!gst_rtp_theora_pay_finish_headers (basepayload))
-        goto header_error;
-      rtptheorapay->need_headers = FALSE;
-    }
-  }
-
-  /* there is a config request, see if we need to insert it */
-  if (keyframe && (rtptheorapay->config_interval > 0) &&
-      rtptheorapay->config_data) {
-    gboolean send_config = FALSE;
-
-    if (rtptheorapay->last_config != -1) {
-      guint64 diff;
-
-      GST_LOG_OBJECT (rtptheorapay,
-          "now %" GST_TIME_FORMAT ", last VOP-I %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (timestamp), GST_TIME_ARGS (rtptheorapay->last_config));
-
-      /* calculate diff between last config in milliseconds */
-      if (timestamp > rtptheorapay->last_config) {
-        diff = timestamp - rtptheorapay->last_config;
-      } else {
-        diff = 0;
-      }
-
-      GST_DEBUG_OBJECT (rtptheorapay,
-          "interval since last config %" GST_TIME_FORMAT, GST_TIME_ARGS (diff));
-
-      /* bigger than interval, queue config */
-      /* FIXME should convert timestamps to running time */
-      if (GST_TIME_AS_SECONDS (diff) >= rtptheorapay->config_interval) {
-        GST_DEBUG_OBJECT (rtptheorapay, "time to send config");
-        send_config = TRUE;
-      }
-    } else {
-      /* no known previous config time, send now */
-      GST_DEBUG_OBJECT (rtptheorapay, "no previous config time, send now");
-      send_config = TRUE;
-    }
-
-    if (send_config) {
-      /* we need to send config now first */
-      /* different TDT type forces flush */
-      gst_rtp_theora_pay_payload_buffer (rtptheorapay, 1,
-          rtptheorapay->config_data, rtptheorapay->config_size,
-          timestamp, GST_CLOCK_TIME_NONE, rtptheorapay->config_extra_len);
-
-      if (timestamp != -1) {
-        rtptheorapay->last_config = timestamp;
-      }
-    }
-  }
-
-  ret = gst_rtp_theora_pay_payload_buffer (rtptheorapay, TDT, data, size,
-      timestamp, duration, 0);
   gst_buffer_unref (buffer);
 
 done:
@@ -747,7 +631,7 @@ done:
 wrong_size:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
-        ("Invalid packet size (%d <= 0xffff)", size), (NULL));
+        ("Invalid packet size (1 < %d <= 0xffff)", size), (NULL));
     gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
@@ -772,22 +656,6 @@ header_error:
   }
 }
 
-static gboolean
-gst_rtp_theora_pay_handle_event (GstPad * pad, GstEvent * event)
-{
-  GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (GST_PAD_PARENT (pad));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_rtp_theora_pay_clear_packet (rtptheorapay);
-      break;
-    default:
-      break;
-  }
-  /* false to let parent handle event as well */
-  return FALSE;
-}
-
 static GstStateChangeReturn
 gst_rtp_theora_pay_change_state (GstElement * element,
     GstStateChange transition)
@@ -815,43 +683,9 @@ gst_rtp_theora_pay_change_state (GstElement * element,
   return ret;
 }
 
-static void
-gst_rtp_theora_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraPay *rtptheorapay;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (object);
-
-  switch (prop_id) {
-    case PROP_CONFIG_INTERVAL:
-      rtptheorapay->config_interval = g_value_get_uint (value);
-      break;
-    default:
-      break;
-  }
-}
-
-static void
-gst_rtp_theora_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraPay *rtptheorapay;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (object);
-
-  switch (prop_id) {
-    case PROP_CONFIG_INTERVAL:
-      g_value_set_uint (value, rtptheorapay->config_interval);
-      break;
-    default:
-      break;
-  }
-}
-
 gboolean
 gst_rtp_theora_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtptheorapay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_THEORA_PAY);
+      GST_RANK_NONE, GST_TYPE_RTP_THEORA_PAY);
 }
