@@ -21,13 +21,9 @@
  * SECTION:element-multiupdsink
  * @see_also: udpsink, multifdsink
  *
- * <refsect2>
- * <para>
  * multiudpsink is a network sink that sends UDP packets to multiple
  * clients.
  * It can be combined with rtp payload encoders to implement RTP streaming.
- * </para>
- * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -371,9 +367,10 @@ static GstFlowReturn
 gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
 {
   GstMultiUDPSink *sink;
-  gint ret, size, num = 0;
+  gint ret, size, num = 0, no_clients = 0;
   guint8 *data;
   GList *clients;
+  gint len;
 
   sink = GST_MULTIUDPSINK (bsink);
 
@@ -391,22 +388,30 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
     GstUDPClient *client;
 
     client = (GstUDPClient *) clients->data;
-    num++;
+    no_clients++;
     GST_LOG_OBJECT (sink, "sending %d bytes to client %p", size, client);
 
     while (TRUE) {
-      ret = sendto (*client->sock, data, size, 0,
-          (struct sockaddr *) &client->theiraddr, sizeof (client->theiraddr));
+      len = gst_udp_get_sockaddr_length (&client->theiraddr);
+
+      ret = sendto (*client->sock,
+#ifdef G_OS_WIN32
+          (char *) data,
+#else
+          data,
+#endif
+          size, 0, (struct sockaddr *) &client->theiraddr, len);
 
       if (ret < 0) {
-        /* we get a non-posix EPERM on Linux when a firewall rule blocks this
-         * destination. We will simply ignore this. */
-        if (errno == EPERM)
-          break;
+        /* some error, just warn, it's likely recoverable and we don't want to
+         * break streaming. We break so that we stop retrying for this client. */
         if (errno != EINTR && errno != EAGAIN) {
-          goto send_error;
+          GST_WARNING_OBJECT (sink, "client %p gave error %d (%s)", client,
+              errno, g_strerror (errno));
+          break;
         }
       } else {
+        num++;
         client->bytes_sent += ret;
         client->packets_sent++;
         sink->bytes_served += ret;
@@ -416,20 +421,10 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   }
   g_mutex_unlock (sink->client_lock);
 
-  GST_LOG_OBJECT (sink, "sent %d bytes to %d clients", size, num);
+  GST_LOG_OBJECT (sink, "sent %d bytes to %d (of %d) clients", size, num,
+      no_clients);
 
   return GST_FLOW_OK;
-
-  /* ERRORS */
-send_error:
-  {
-    /* if sendto returns an error, something is seriously wrong */
-    g_mutex_unlock (sink->client_lock);
-    GST_DEBUG_OBJECT (sink, "got send error %d: %s", errno, g_strerror (errno));
-    GST_ELEMENT_ERROR (sink, STREAM, FAILED, (NULL),
-        ("Got send error %d: %s", errno, g_strerror (errno)));
-    return GST_FLOW_ERROR;
-  }
 }
 
 static void
@@ -607,7 +602,6 @@ static gboolean
 gst_multiudpsink_init_send (GstMultiUDPSink * sink)
 {
   guint bc_val;
-  gint ret;
   GList *clients;
   GstUDPClient *client;
 
@@ -625,9 +619,8 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   }
 
   bc_val = 1;
-  if ((ret =
-          setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
-              sizeof (bc_val))) < 0)
+  if (setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
+          sizeof (bc_val)) < 0)
     goto no_broadcast;
 
   sink->bytes_to_serve = 0;
@@ -640,7 +633,7 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   for (clients = sink->clients; clients; clients = g_list_next (clients)) {
     client = (GstUDPClient *) clients->data;
     if (sink->auto_multicast && gst_udp_is_multicast (&client->theiraddr))
-      gst_udp_join_group (*(client->sock), &client->theiraddr);
+      gst_udp_join_group (*(client->sock), &client->theiraddr, NULL);
   }
   return TRUE;
 
@@ -692,7 +685,7 @@ gst_multiudpsink_add_internal (GstMultiUDPSink * sink, const gchar * host,
       GST_DEBUG_OBJECT (sink, "multicast address detected");
       if (sink->auto_multicast) {
         GST_DEBUG_OBJECT (sink, "joining multicast group");
-        gst_udp_join_group (*(client->sock), &client->theiraddr);
+        gst_udp_join_group (*(client->sock), &client->theiraddr, NULL);
       }
     } else {
       GST_DEBUG_OBJECT (sink, "normal address detected");
