@@ -36,7 +36,6 @@
  * upstream, which I'll consider doing later on. If you get compiler
  * errors here, check your linux/time.h && sys/time.h header setup.
  */
-#include <sys/ioctl.h>
 #include <sys/types.h>
 #ifndef __sun
 #include <linux/types.h>
@@ -49,20 +48,9 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstpushsrc.h>
-#include <gst/controller/gstcontroller.h>
 
 #include <gst/interfaces/propertyprobe.h>
 
-
-/* size of v4l2 buffer pool in streaming case */
-#define GST_V4L2_MAX_BUFFERS 16
-#define GST_V4L2_MIN_BUFFERS 1
-
-/* max frame width/height */
-#define GST_V4L2_MAX_SIZE (1<<15) /* 2^15 == 32768 */
-
-
-
 G_BEGIN_DECLS
 
 #define GST_V4L2_OBJECT(obj) (GstV4l2Object *)(obj)
@@ -84,13 +72,10 @@ struct _GstV4l2Object {
   /* the video-device's file descriptor */
   gint video_fd;
   GstPoll * poll;
-  gboolean can_poll_device;
 
   /* the video buffer (mmap()'ed) */
   guint8 **buffer;
 
-  enum v4l2_buf_type type;   /* V4L2_BUF_TYPE_VIDEO_CAPTURE, V4L2_BUF_TYPE_VIDEO_OUTPUT */
-
   /* the video device's capabilities */
   struct v4l2_capability vcap;
 
@@ -101,14 +86,12 @@ struct _GstV4l2Object {
   struct v4l2_input vinput;
 
   /* lists... */
-  GSList *formats;              /* list of available capture formats */
-
   GList *colors;
   GList *norms;
   GList *channels;
 
   /* properties */
-  v4l2_std_id tv_norm;
+  gchar *norm;
   gchar *channel;
   gulong frequency;
 
@@ -133,25 +116,17 @@ GType gst_v4l2_object_get_type (void);
     PROP_DEVICE,			\
     PROP_DEVICE_NAME,			\
     PROP_DEVICE_FD,			\
-    PROP_FLAGS,                         \
-    PROP_BRIGHTNESS,			\
-    PROP_CONTRAST,			\
-    PROP_SATURATION,			\
-    PROP_HUE,                           \
-    PROP_TV_NORM
+    PROP_FLAGS
 
 /* create/destroy */
 GstV4l2Object *	gst_v4l2_object_new 		 (GstElement * element,
-                                                  enum v4l2_buf_type  type,
-                                                  const char *default_device,
                    				  GstV4l2GetInOutFunction get_in_out_func,
                    				  GstV4l2SetInOutFunction set_in_out_func,
 		   				  GstV4l2UpdateFpsFunction   update_fps_func);
 void 	        gst_v4l2_object_destroy 	 (GstV4l2Object * v4l2object);
 
 /* properties */
-
-void 	  gst_v4l2_object_install_properties_helper (GObjectClass *gobject_class, const char *default_device);
+void 	  gst_v4l2_object_install_properties_helper (GObjectClass *gobject_class);
 
 gboolean  gst_v4l2_object_set_property_helper       (GstV4l2Object *v4l2object,
 				                     guint prop_id, const GValue * value,
@@ -176,26 +151,6 @@ GValueArray* gst_v4l2_probe_get_values      (GstPropertyProbe * probe, guint pro
                                              const GParamSpec * pspec,
                                              GList ** klass_devices);
 
-GstCaps*      gst_v4l2_object_probe_caps_for_format (GstV4l2Object *v4l2object, guint32 pixelformat,
-                                             const GstStructure * template);
-
-gboolean      gst_v4l2_object_get_caps_info (GstV4l2Object *v4l2object, GstCaps *caps,
-                                             struct v4l2_fmtdesc **format, gint *w, gint *h,
-                                             gboolean * interlaced, guint *fps_n, guint *fps_d, guint *size);
-
-
-GSList*       gst_v4l2_object_get_format_list  (GstV4l2Object *v4l2object);
-
-GstCaps*      gst_v4l2_object_get_all_caps (void);
-
-GstStructure* gst_v4l2_object_v4l2fourcc_to_structure (guint32 fourcc);
-
-gboolean      gst_v4l2_object_set_format (GstV4l2Object *v4l2object, guint32 pixelformat, guint32 width, guint32 height, gboolean interlaced);
-
-gboolean      gst_v4l2_object_start_streaming (GstV4l2Object *v4l2object);
-gboolean      gst_v4l2_object_stop_streaming (GstV4l2Object *v4l2object);
-
-
 #define GST_IMPLEMENT_V4L2_PROBE_METHODS(Type_Class, interface_as_function)                 \
                                                                                             \
 static void                                                                                 \
